@@ -86,10 +86,15 @@ static void _osync_client_connect_callback(void *data, OSyncError *error)
 	OSyncMessage *message = baton->message;
 	OSyncClient *client = baton->client;
 
+	OSyncObjTypeSink *sink = osync_plugin_info_get_sink(client->plugin_info);
+	int slowsync = osync_objtype_sink_get_slowsync(sink);
+
 	OSyncMessage *reply = NULL;
 	if (!osync_error_is_set(&error)) {
 		reply = osync_message_new_reply(message, &locerror);
 		//Send connect specific reply data
+		osync_message_write_int(reply, slowsync);
+
 	} else {
 		reply = osync_message_new_errorreply(message, error, &locerror);
 	}
@@ -635,6 +640,7 @@ static osync_bool _osync_client_handle_discover(OSyncClient *client, OSyncMessag
 		osync_message_write_int(reply, 1);
 		osync_message_write_string(reply, osync_version_get_plugin(version));
 		osync_message_write_string(reply, osync_version_get_priority(version));
+		osync_message_write_string(reply, osync_version_get_vendor(version));
 		osync_message_write_string(reply, osync_version_get_modelversion(version));
 		osync_message_write_string(reply, osync_version_get_firmwareversion(version));
 		osync_message_write_string(reply, osync_version_get_softwareversion(version));
@@ -701,6 +707,9 @@ static osync_bool _osync_client_handle_connect(OSyncClient *client, OSyncMessage
 		if (!reply)
 			goto error;
 		
+		/* SlowSync dummy value for connect reply message handler. */
+		osync_message_write_int(reply, FALSE);
+
 		if (!osync_queue_send_message(client->outgoing, NULL, reply, error))
 			goto error_free_reply;
 		
@@ -792,11 +801,13 @@ static osync_bool _osync_client_handle_get_changes(OSyncClient *client, OSyncMes
 {
 	osync_trace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, client, message, error);
 
+	int slowsync;
 	char *objtype = NULL;
 	OSyncMessage *reply = NULL;
 	
 	osync_message_read_string(message, &objtype);
-	osync_trace(TRACE_INTERNAL, "Searching sink for %s", objtype);
+	osync_message_read_int(message, &slowsync);
+	osync_trace(TRACE_INTERNAL, "Searching sink for %s (slowsync: %i)", objtype, slowsync);
 	
 	OSyncObjTypeSink *sink = NULL;
 	if (objtype) {
@@ -822,6 +833,13 @@ static osync_bool _osync_client_handle_get_changes(OSyncClient *client, OSyncMes
 		
 		osync_message_unref(reply);
 	} else {
+		/* set slowsync (again) if the slow-sync got requested during the connect() call
+		   of a member - and not during frontend/engine itself (e.g. anchor mismatch). */
+		if (slowsync)
+			osync_objtype_sink_set_slowsync(sink, TRUE);
+		else
+			osync_objtype_sink_set_slowsync(sink, FALSE);
+
 		OSyncContext *context = _create_context(client, message, _osync_client_get_changes_callback, NULL, error);
 		if (!context)
 			goto error;
@@ -1058,7 +1076,7 @@ static void _osync_client_message_handler(OSyncMessage *message, void *user_data
 
 	OSyncError *error = NULL;
 
-	osync_trace(TRACE_INTERNAL, "plugin received command %i", osync_message_get_command(message));
+	osync_trace(TRACE_INTERNAL, "plugin received command %i (%s)", osync_message_get_command(message), osync_message_get_commandstr(message));
 
 	switch (osync_message_get_command(message)) {
 		case OSYNC_MESSAGE_NOOP:
@@ -1236,11 +1254,13 @@ OSyncClient *osync_client_new(OSyncError **error)
 	return client;
 }
 
-void osync_client_ref(OSyncClient *client)
+OSyncClient *osync_client_ref(OSyncClient *client)
 {
 	osync_assert(client);
 	
 	g_atomic_int_inc(&(client->ref_count));
+
+	return client;
 }
 
 void osync_client_unref(OSyncClient *client)
@@ -1262,7 +1282,7 @@ void osync_client_unref(OSyncClient *client)
 				osync_queue_disconnect(client->outgoing, NULL);
 			osync_queue_free(client->outgoing);
 		}
-	
+		
 		if (client->plugin)
 			osync_plugin_unref(client->plugin);
 		
