@@ -34,11 +34,12 @@
 #include "opensync_client_proxy_internals.h"
 
 #include <sys/types.h>
+#include <sys/wait.h>
 #include <signal.h>
 
 #ifdef _WIN32
 /* Historical signals specified by POSIX. */
-#define SIGKILL 9       /* Kill (cannot be blocked, caught, or ignored).  */
+#define SIGKILL 9   /* Kill (cannot be blocked, caught, or ignored).  */
 #define SIGTERM 15	/* can be caught and interpreted or ignored by the process */
 #endif //_WIN32
 
@@ -77,7 +78,7 @@ typedef struct callContext {
 } callContext;
 
 //portable kill pid helper
-static int _osync_kill(pid_t pid, int sig) 
+/*static int _osync_kill(pid_t pid, int sig) 
 {
 #ifndef _WIN32
 	return kill(pid, sig);
@@ -104,12 +105,12 @@ end:
 	CloseHandle(hProc);
 	return (ret);
 #endif //_WIN32
-} 
+}*/
 
-static char *_osync_client_pid_filename(OSyncClientProxy *proxy)
+/*static char *_osync_client_pid_filename(OSyncClientProxy *proxy)
 {
 	return g_strdup_printf("%s/osplugin.pid", proxy->path);
-}
+}*/
 
 /*static osync_bool osync_client_remove_pidfile(OSyncClientProxy *proxy, OSyncError **error)
 {
@@ -123,9 +124,9 @@ static char *_osync_client_pid_filename(OSyncClientProxy *proxy)
 	
 	g_free(pidpath);
 	return TRUE;
-}
+}*/
 
-static osync_bool _osync_client_create_pidfile(OSyncClientProxy *proxy, OSyncError **error)
+/*static osync_bool _osync_client_create_pidfile(OSyncClientProxy *proxy, OSyncError **error)
 {
 	char *pidpath = _osync_client_pid_filename(proxy);
 	char *pidstr = g_strdup_printf("%ld", (long)proxy->child_pid);
@@ -141,7 +142,7 @@ static osync_bool _osync_client_create_pidfile(OSyncClientProxy *proxy, OSyncErr
 	return TRUE;
 }*/
 
-static osync_bool _osync_client_kill_old_osplugin(OSyncClientProxy *proxy, OSyncError **error)
+/*static osync_bool _osync_client_kill_old_osplugin(OSyncClientProxy *proxy, OSyncError **error)
 {
 	osync_bool ret = FALSE;
 
@@ -151,7 +152,7 @@ static osync_bool _osync_client_kill_old_osplugin(OSyncClientProxy *proxy, OSync
 
 	char *pidpath = _osync_client_pid_filename(proxy);
 
-	/* Simply returns if there is no PID file */
+	// Simply returns if there is no PID file
 	if (!g_file_test(pidpath, G_FILE_TEST_EXISTS)) {
 		ret = TRUE;
 		goto out_free_path;
@@ -168,7 +169,7 @@ static osync_bool _osync_client_kill_old_osplugin(OSyncClientProxy *proxy, OSync
 
 	if (_osync_kill(pid, SIGTERM) < 0) {
 		osync_trace(TRACE_INTERNAL, "Error killing old osplugin: %s. Stale pid file?", g_strerror(errno));
-		/* Don't return failure if kill() failed, because it may be a stale pid file */
+		// Don't return failure if kill() failed, because it may be a stale pid file
 	}
 
 	int count = 0;
@@ -179,7 +180,7 @@ static osync_bool _osync_client_kill_old_osplugin(OSyncClientProxy *proxy, OSync
 			break;
 		}
 		osync_trace(TRACE_INTERNAL, "Waiting for other side to terminate");
-		/*FIXME: Magic numbers are evil */
+		// FIXME: Magic numbers are evil
 		g_usleep(500000);
 	}
 
@@ -188,7 +189,7 @@ static osync_bool _osync_client_kill_old_osplugin(OSyncClientProxy *proxy, OSync
 		goto out_free_str;
 	}
 
-	/* Success */
+	// Success
 	ret = TRUE;
 
 out_free_str:
@@ -197,7 +198,8 @@ out_free_path:
 	g_free(pidpath);
 //out:
 	return ret;
-}
+}*/
+
 
 /** This function takes care of the messages received on the outgoing (sending)
  * queue. The only messages we can receive there, are HUPs or ERRORs. */
@@ -318,18 +320,11 @@ static void _osync_client_proxy_discover_handler(OSyncMessage *message, void *us
 			OSyncObjTypeSink *sink = NULL;
 			if (!osync_demarshal_objtype_sink(message, &sink, &locerror))
 				goto error;
-			osync_trace(TRACE_INTERNAL, "Received sink: %s", osync_objtype_sink_get_name(sink));
-	
+
 			proxy->objtypes = g_list_append(proxy->objtypes, sink);
-			
-			if (proxy->member) {
-				osync_member_add_objtype(proxy->member, osync_objtype_sink_get_name(sink));
-				const OSyncList *f = osync_objtype_sink_get_objformats(sink);
-				for (; f; f = f->next) {
-					const char *format = f->data;
-					osync_member_add_objformat(proxy->member, osync_objtype_sink_get_name(sink), format);
-				}
-			}
+
+			if (proxy->member)
+				osync_member_add_objtype_sink(proxy->member, sink);
 		}
 		
 		/* Merger - Set the capabilities */
@@ -349,6 +344,9 @@ static void _osync_client_proxy_discover_handler(OSyncMessage *message, void *us
 			osync_version_set_priority(version, str);
 			g_free(str);
 			osync_message_read_string(message, &str);
+			osync_version_set_vendor(version, str);
+			g_free(str);
+			osync_message_read_string(message, &str);
 			osync_version_set_modelversion(version, str);
 			g_free(str);
 			osync_message_read_string(message, &str);
@@ -402,6 +400,8 @@ static void _osync_client_proxy_discover_handler(OSyncMessage *message, void *us
 					goto error_free_capabilities; 
  				if (!osync_member_save(member, &locerror)) /* TODO: Merger we have to save? */
  					goto error_free_capabilities;
+
+				osync_capabilities_unref(capabilities);
  			}
  		}
 		if (version)
@@ -437,6 +437,7 @@ error:
 
 static void _osync_client_proxy_connect_handler(OSyncMessage *message, void *user_data)
 {
+	int slowsync;
 	callContext *ctx = user_data;
 	OSyncClientProxy *proxy = ctx->proxy;
 	OSyncError *error = NULL;
@@ -445,10 +446,11 @@ static void _osync_client_proxy_connect_handler(OSyncMessage *message, void *use
 	osync_trace(TRACE_ENTRY, "%s(%p, %p)", __func__, message, user_data);
 	
 	if (osync_message_get_cmd(message) == OSYNC_MESSAGE_REPLY) {
-		ctx->connect_callback(proxy, ctx->connect_callback_data, NULL);
+		osync_message_read_int(message, &slowsync);
+		ctx->connect_callback(proxy, ctx->connect_callback_data, slowsync, NULL);
 	} else if (osync_message_get_cmd(message) == OSYNC_MESSAGE_ERRORREPLY) {
 		osync_demarshal_error(message, &error);
-		ctx->connect_callback(proxy, ctx->connect_callback_data, error);
+		ctx->connect_callback(proxy, ctx->connect_callback_data, FALSE, error);
 		osync_error_unref(&error);
 	} else {
 		osync_error_set(&locerror, OSYNC_ERROR_GENERIC, "Unexpected reply");
@@ -721,11 +723,13 @@ error:
 	return NULL;
 }
 
-void osync_client_proxy_ref(OSyncClientProxy *proxy)
+OSyncClientProxy *osync_client_proxy_ref(OSyncClientProxy *proxy)
 {
 	osync_assert(proxy);
 	
 	g_atomic_int_inc(&(proxy->ref_count));
+
+	return proxy;
 }
 
 void osync_client_proxy_unref(OSyncClientProxy *proxy)
@@ -797,6 +801,9 @@ osync_bool osync_client_proxy_spawn(OSyncClientProxy *proxy, OSyncStartType type
 		if (!osync_queue_new_pipes(&read2, &write2, error))
 			goto error_free_pipe1;
 		
+		proxy->outgoing = write1;
+		proxy->incoming = read2;
+		
 		/* Now we either spawn a new process, or we create a new thread */
 		if (type == OSYNC_START_TYPE_THREAD) {
 			proxy->client = osync_client_new(error);
@@ -819,50 +826,57 @@ osync_bool osync_client_proxy_spawn(OSyncClientProxy *proxy, OSyncStartType type
 				goto error_free_pipe2;
 		} else {
 			/* First lets see if the old plugin exists, and kill it if it does */
-			if (!_osync_client_kill_old_osplugin(proxy, error))
-				goto error;
-			
-			/*todo*/
-			
-			
-		
-		/*if (!osync_queue_exists(proxy->outgoing) || !osync_queue_is_alive(proxy->outgoing)) {
-			pid_t cpid = fork();
-			if (cpid == 0) {
-				osync_trace_reset_indent();
-	
-				osync_trace(TRACE_INTERNAL, "About to exec osplugin");
-				char *memberstring = g_strdup_printf("%lli", osync_member_get_id(proxy->member));
-				execlp("osplugin", "osplugin", osync_group_get_configdir(engine->group), memberstring, NULL);
-				
-				if (errno == ENOENT) {
-					osync_trace(TRACE_INTERNAL, "Unable to find osplugin. Trying local path.");
-					execlp("./osplugin", "osplugin", osync_group_get_configdir(engine->group), memberstring, NULL);
+			//if (!_osync_client_kill_old_osplugin(proxy, error))
+			//	goto error;
+
+			//if (!osync_queue_exists(proxy->outgoing) || !osync_queue_is_alive(proxy->outgoing)) {
+			if (!proxy->outgoing || !osync_queue_exists(proxy->outgoing) || !osync_queue_is_alive(proxy->outgoing)) {
+				pid_t cpid = fork();
+				if (cpid == 0) {
+					osync_trace_reset_indent();
+					
+					/* close the read and write ends of the pipes */
+					osync_queue_disconnect(write1, error);
+					osync_queue_disconnect(read2, error);
+						
+					osync_trace(TRACE_INTERNAL, "About to exec osplugin");
+					//char *memberstring = g_strdup_printf("%lli", osync_member_get_id(proxy->member));
+					//execlp("osplugin", "osplugin", osync_group_get_configdir(osync_member_get_group(osync_proxy_get_member(proxy)), memberstring, NULL);
+					char *readfd = g_strdup_printf("%i", osync_queue_get_fd(read1));
+					char *writefd = g_strdup_printf("%i", osync_queue_get_fd(write2));
+					execlp(OSPLUGIN, "osplugin", "-f", readfd, writefd, NULL);
+
+					if (errno == ENOENT) {
+						osync_trace(TRACE_INTERNAL, "Unable to find osplugin. Trying local path.");
+						//execlp("osplugin", "osplugin", osync_group_get_configdir(osync_member_get_group(osync_proxy_get_member(proxy)), memberstring, NULL);
+						execlp("./osplugin", "osplugin", "-f", readfd, writefd, NULL);
+					}
+											
+					osync_trace(TRACE_INTERNAL, strerror(errno));
+					osync_trace(TRACE_INTERNAL, "Unable to execute osplugin.");
+					exit(1);
+				} else {
+					/* close the read and write ends of the pipes */
+					osync_queue_disconnect(write2, error);
+					osync_queue_disconnect(read1, error);
 				}
-				
-				osync_trace(TRACE_INTERNAL, "unable to exec");
-				exit(1);
+	
+				proxy->child_pid = cpid;
+			
+				//while (!osync_queue_exists(proxy->outgoing)) {
+				//	osync_trace(TRACE_INTERNAL, "Waiting for other side to create fifo");
+				//	g_usleep(500000);
+				//}
+			
+				osync_trace(TRACE_INTERNAL, "Queue was created");
 			}
 	
-			proxy->child_pid = cpid;
-			
-			while (!osync_queue_exists(proxy->outgoing)) {
-				osync_trace(TRACE_INTERNAL, "Waiting for other side to create fifo");
-				g_usleep(500000);
-			}
-			
-			osync_trace(TRACE_INTERNAL, "Queue was created");
-		}
-	
-		if (proxy->child_pid) {
-			if (!_osync_client_create_pidfile(proxy, error))
-				goto error;
-		}*/
+			//if (proxy->child_pid) {
+			//	if (!_osync_client_create_pidfile(proxy, error))
+			//		goto error;
+			//}
 		}
 		
-		proxy->outgoing = write1;
-		proxy->incoming = read2;
-	
 		/* We now connect to our incoming queue */
 		if (!osync_queue_connect(proxy->incoming, OSYNC_QUEUE_RECEIVER, error))
 			goto error;
@@ -931,7 +945,6 @@ osync_bool osync_client_proxy_shutdown(OSyncClientProxy *proxy, OSyncError **err
 		osync_message_unref(message);
 		goto error;
 	}
-	
 	osync_message_unref(message);
 	
 	/* After we received the HUP, we can disconnect */
@@ -943,26 +956,25 @@ osync_bool osync_client_proxy_shutdown(OSyncClientProxy *proxy, OSyncError **err
 		
 		osync_client_unref(proxy->client);
 	} else if (proxy->type == OSYNC_START_TYPE_PROCESS) {
-		/*if (client->child_pid) {
-		int status;
-		if (waitpid(client->child_pid, &status, 0) == -1) {
-			osync_error_set(error, OSYNC_ERROR_GENERIC, "Error waiting for osplugin process: %s", g_strerror(errno));
-			goto error;
+		if (proxy->child_pid) {
+			int status;
+			if (waitpid(proxy->child_pid, &status, 0) == -1) {
+				osync_error_set(error, OSYNC_ERROR_GENERIC, "Error waiting for osplugin process: %s", g_strerror(errno));
+				goto error;
+			}
+
+			if (!WIFEXITED(status))
+				osync_trace(TRACE_INTERNAL, "Child has exited abnormally");
+			else if (WEXITSTATUS(status) != 0)
+				osync_trace(TRACE_INTERNAL, "Child has returned non-zero exit status (%d)", WEXITSTATUS(status));
+
+			//if (!osync_client_remove_pidfile(client, error))
+			//	goto error;
 		}
-
-		if (!WIFEXITED(status))
-			osync_trace(TRACE_INTERNAL, "Child has exited abnormally");
-		else if (WEXITSTATUS(status) != 0)
-			osync_trace(TRACE_INTERNAL, "Child has returned non-zero exit status (%d)", WEXITSTATUS(status));
-
-		if (!osync_client_remove_pidfile(client, error))
-			goto error;
-	}*/
-		
 		
 		/* First lets see if the old plugin exists, and kill it if it does */
-		if (!_osync_client_kill_old_osplugin(proxy, error))
-			goto error;
+		//if (!_osync_client_kill_old_osplugin(proxy, error))
+		//	goto error;
 	}
 			
 	osync_queue_free(proxy->incoming);
@@ -1104,7 +1116,7 @@ OSyncObjTypeSink *osync_client_proxy_nth_objtype(OSyncClientProxy *proxy, int nt
 
 osync_bool osync_client_proxy_connect(OSyncClientProxy *proxy, connect_cb callback, void *userdata, const char *objtype, osync_bool slowsync, OSyncError **error)
 {
-	osync_trace(TRACE_ENTRY, "%s(%p, %p, %p, %s, %p)", __func__, proxy, callback, userdata, objtype, error);
+	osync_trace(TRACE_ENTRY, "%s(%p, %p, %p, %s, %i, %p)", __func__, proxy, callback, userdata, objtype, slowsync, error);
 	
 	callContext *ctx = osync_try_malloc0(sizeof(callContext), error);
 	if (!ctx)
@@ -1209,9 +1221,9 @@ error:
 	return FALSE;
 }
 
-osync_bool osync_client_proxy_get_changes(OSyncClientProxy *proxy, get_changes_cb callback, void *userdata, const char *objtype, OSyncError **error)
+osync_bool osync_client_proxy_get_changes(OSyncClientProxy *proxy, get_changes_cb callback, void *userdata, const char *objtype, osync_bool slowsync, OSyncError **error)
 {
-	osync_trace(TRACE_ENTRY, "%s(%p, %p, %p, %s, %p)", __func__, proxy, callback, userdata, objtype, error);
+	osync_trace(TRACE_ENTRY, "%s(%p, %p, %p, %s, %i, %p)", __func__, proxy, callback, userdata, objtype, slowsync, error);
 	
 	callContext *ctx = osync_try_malloc0(sizeof(callContext), error);
 	if (!ctx)
@@ -1228,6 +1240,7 @@ osync_bool osync_client_proxy_get_changes(OSyncClientProxy *proxy, get_changes_c
 	osync_message_set_handler(message, _osync_client_proxy_get_changes_handler, ctx);
 
 	osync_message_write_string(message, objtype);
+	osync_message_write_int(message, slowsync);
 	
 	if (!osync_queue_send_message(proxy->outgoing, proxy->incoming, message, error))
 		goto error_free_message;
