@@ -19,6 +19,7 @@
  */
  
 #include "opensync.h"
+#include "opensync_xml.h"
 #include "opensync_internals.h"
 
 #include "opensync-format.h"
@@ -26,13 +27,10 @@
 #include "opensync_group_internals.h"
 #include "opensync-db.h"
 
-
-#include "opensync_xml.h"
-
 #ifndef _WIN32
 #include <sys/file.h>
 
-#ifdef NOT_HAVE_FLOCK
+#ifndef HAVE_FLOCK
 #define LOCK_SH 1
 #define LOCK_EX 2
 #define LOCK_NB 4
@@ -66,62 +64,6 @@ static int flock(int fd, int operation)
 #endif //NOT_HAVE_FLOCK
 #endif //not defined _WIN32
 
-static void _build_list(gpointer key, gpointer value, gpointer user_data)
-{
-	if (GPOINTER_TO_INT(value) >= 2) {
-		GList **l = user_data;
-		*l = g_list_append(*l, key);
-	}
-}
-
-static void _add_one(gpointer key, gpointer value, gpointer user_data)
-{
-	GHashTable *table = user_data;
-	
-	int num = GPOINTER_TO_INT(value);
-	g_hash_table_replace(table, key, GINT_TO_POINTER(num + 1));
-}
-
-static GList *_osync_group_get_supported_objtypes(OSyncGroup *group)
-{
-	GList *m = NULL;
-	GList *ret = NULL;
-	GHashTable *table = g_hash_table_new(g_str_hash, g_str_equal);
-    
-	int num_data = 0;
-	int i;
-    
-	/* Loop over all members... */
-	for (m = group->members; m; m = m->next) {
-		OSyncMember *member = m->data;
-		int num_member = osync_member_num_objtypes(member);
-		/* ... and get the objtype from each of the members. */
-		for (i = 0; i < num_member; i++) {
-			const char *objtype = osync_member_nth_objtype(member, i);
-			if (objtype != NULL) {
-				/* For each objtype, add 1 to the hashtable. If the objtype is
-			 	* the special objtype "data", add 1 to all objtypes */
-				if(!strcmp(objtype, "data"))
-					num_data++;
-				int num = GPOINTER_TO_INT(g_hash_table_lookup(table, objtype));
-				g_hash_table_replace(table, (char *)objtype, GINT_TO_POINTER(num + 1));
-			}
-		}
-	}
-	
-	for (i = 0; i < num_data; i++)
-		g_hash_table_foreach(table, _add_one, table);
-	
-	if (g_hash_table_size(table) == 0 && num_data >= 2) {
-		osync_trace(TRACE_INTERNAL, "No objtype found yet, but data available");
-		g_hash_table_replace(table, "data", GINT_TO_POINTER(num_data));
-	}
-	
-	g_hash_table_foreach(table, _build_list, &ret);
-	g_hash_table_destroy(table);
-	return ret;
-}
-
 /**
  * @defgroup OSyncGroupPrivateAPI OpenSync Group Internals
  * @ingroup OSyncPrivate
@@ -219,6 +161,68 @@ error:
 	return FALSE;
 }
 
+static void _build_list(gpointer key, gpointer value, gpointer user_data)
+{
+	if (GPOINTER_TO_INT(value) >= 2) {
+		GList **l = user_data;
+		*l = g_list_append(*l, key);
+	}
+}
+
+static void _add_one(gpointer key, gpointer value, gpointer user_data)
+{
+	GHashTable *table = user_data;
+	
+	int num = GPOINTER_TO_INT(value);
+	g_hash_table_replace(table, key, GINT_TO_POINTER(num + 1));
+}
+
+/*! @brief Get list of supported object types of the group 
+ * 
+ * @param group The group
+ * @returns List of supported object types 
+ * 
+ */
+static GList *_osync_group_get_supported_objtypes(OSyncGroup *group)
+{
+	GList *m = NULL;
+	GList *ret = NULL;
+	GHashTable *table = g_hash_table_new(g_str_hash, g_str_equal);
+    
+	int num_data = 0;
+	int i;
+    
+	/* Loop over all members... */
+	for (m = group->members; m; m = m->next) {
+		OSyncMember *member = m->data;
+		int num_member = osync_member_num_objtypes(member);
+		/* ... and get the objtype from each of the members. */
+		for (i = 0; i < num_member; i++) {
+			const char *objtype = osync_member_nth_objtype(member, i);
+			if (objtype != NULL) {
+				/* For each objtype, add 1 to the hashtable. If the objtype is
+			 	* the special objtype "data", add 1 to all objtypes */
+				if(!strcmp(objtype, "data"))
+					num_data++;
+				int num = GPOINTER_TO_INT(g_hash_table_lookup(table, objtype));
+				g_hash_table_replace(table, (char *)objtype, GINT_TO_POINTER(num + 1));
+			}
+		}
+	}
+	
+	for (i = 0; i < num_data; i++)
+		g_hash_table_foreach(table, _add_one, table);
+	
+	if (g_hash_table_size(table) == 0 && num_data >= 2) {
+		osync_trace(TRACE_INTERNAL, "No objtype found yet, but data available");
+		g_hash_table_replace(table, "data", GINT_TO_POINTER(num_data));
+	}
+	
+	g_hash_table_foreach(table, _build_list, &ret);
+	g_hash_table_destroy(table);
+	return ret;
+}
+
 /*@}*/
 
 /**
@@ -246,6 +250,12 @@ OSyncGroup *osync_group_new(OSyncError **error)
 	if (!group)
 		goto error;
 	group->ref_count = 1;
+
+	/* By default Merger is enabled */
+	group->merger_enabled = TRUE;
+
+	/* By default Converter is enabled */
+	group->converter_enabled = TRUE;
 	
 	osync_trace(TRACE_EXIT, "%s: %p", __func__, group);
 	return group;
@@ -255,13 +265,26 @@ error:
 	return NULL;
 }
 
-void osync_group_ref(OSyncGroup *group)
+
+/** @brief Increase the reference count of the group
+ * 
+ * @param group The group
+ * 
+ */
+OSyncGroup *osync_group_ref(OSyncGroup *group)
 {
 	osync_assert(group);
 	
 	g_atomic_int_inc(&(group->ref_count));
+
+	return group;
 }
 
+/** @brief Decrease the reference count of the group
+ * 
+ * @param group The group
+ * 
+ */
 void osync_group_unref(OSyncGroup *group)
 {
 	osync_assert(group);
@@ -480,6 +503,7 @@ osync_bool osync_group_save(OSyncGroup *group, OSyncError **error)
 	doc = xmlNewDoc((xmlChar*)"1.0");
 	doc->children = xmlNewDocNode(doc, NULL, (xmlChar*)"syncgroup", NULL);
 	
+	// TODO: reimplement the filter!
 	//The filters
 	/*GList *f;
 	for (f = group->filters; f; f = f->next) {
@@ -519,7 +543,11 @@ osync_bool osync_group_save(OSyncGroup *group, OSyncError **error)
 	xmlNewChild(doc->children, NULL, (xmlChar*)"last_sync", (xmlChar*)tmstr);
 	g_free(tmstr);
 
-	xmlSaveFile(filename, doc);
+	xmlNewChild(doc->children, NULL, (xmlChar*)"merger_enabled", (xmlChar*) (group->merger_enabled ? "true" : "false"));
+	xmlNewChild(doc->children, NULL, (xmlChar*)"converter_enabled", (xmlChar*) (group->converter_enabled ? "true" : "false"));
+
+
+	xmlSaveFormatFile(filename, doc, 1);
 	xmlFreeDoc(doc);
 	g_free(filename);
 
@@ -565,7 +593,7 @@ osync_bool osync_group_delete(OSyncGroup *group, OSyncError **error)
 	return TRUE;
 }
 
-/*! @brief Reset all databases of a group (anchor, hashtable and archieve) 
+/*! @brief Reset all databases of a group (anchor, hashtable and archive) 
  * 
  * @param group The group
  * @param error Pointer to an error struct
@@ -586,7 +614,7 @@ osync_bool osync_group_reset(OSyncGroup *group, OSyncError **error)
 	for (m = group->members; m; m = m->next) {
 		OSyncMember *member = m->data;
 
-		// flush hashtable...
+		/* flush hashtable */
 		path = g_strdup_printf("%s/hashtable.db", osync_member_get_configdir(member));
 		if (!(db = osync_db_new(error)))
 			goto error_and_free;
@@ -598,7 +626,7 @@ osync_bool osync_group_reset(OSyncGroup *group, OSyncError **error)
 
 		g_free(path);
 
-		// flush anchor db ... 
+		/* flush anchor db */ 
 		path = g_strdup_printf("%s/anchor.db", osync_member_get_configdir(member));
 		if (!(db = osync_db_new(error)))
 			goto error_and_free;
@@ -667,7 +695,7 @@ osync_bool osync_group_load(OSyncGroup *group, const char *path, OSyncError **er
 	filename = g_strdup_printf("%s/syncgroup.conf", real_path);
 	g_free(real_path);
 	
-	if (!osync_open_xml_file(&doc, &cur, filename, "syncgroup", error)) {
+	if (!osync_xml_open_file(&doc, &cur, filename, "syncgroup", error)) {
 		g_free(filename);
 		goto error;
 	}
@@ -681,7 +709,21 @@ osync_bool osync_group_load(OSyncGroup *group, const char *path, OSyncError **er
 	
 			if (!xmlStrcmp(cur->name, (const xmlChar *)"last_sync"))
 				group->last_sync = (time_t)atoi(str);
-	
+			
+			//TODO: remove the next 2 lines later
+			if (!xmlStrcmp(cur->name, (const xmlChar *)"enable_merger"))
+				group->merger_enabled = (!g_ascii_strcasecmp("true", str)) ? TRUE : FALSE;
+			//TODO: remove the next 2 lines later
+			if (!xmlStrcmp(cur->name, (const xmlChar *)"enable_converter"))
+				group->converter_enabled = (!g_ascii_strcasecmp("true", str)) ? TRUE : FALSE;
+
+			if (!xmlStrcmp(cur->name, (const xmlChar *)"merger_enabled"))
+				group->merger_enabled = (!g_ascii_strcasecmp("true", str)) ? TRUE : FALSE;
+
+			if (!xmlStrcmp(cur->name, (const xmlChar *)"converter_enabled"))
+				group->converter_enabled = (!g_ascii_strcasecmp("true", str)) ? TRUE : FALSE;
+
+			// TODO: reimplement the filter!
 			/*if (!xmlStrcmp(cur->name, (const xmlChar *)"filter")) {
 				filternode = cur->xmlChildrenNode;
 				OSyncFilter *filter = osync_filter_new();
@@ -750,7 +792,7 @@ osync_bool osync_group_load(OSyncGroup *group, const char *path, OSyncError **er
 	}
 	xmlFreeDoc(doc);
 	
-	//Check for sanity
+	/* Check for sanity */
 	if (!group->name) {
 		osync_error_set(error, OSYNC_ERROR_MISCONFIGURATION, "Loaded a group without a name");
 		goto error;
@@ -878,6 +920,12 @@ void osync_group_set_configdir(OSyncGroup *group, const char *directory)
 	group->configdir = g_strdup(directory);
 }
 
+/*! @brief The number of object types of the group
+ * 
+ * @param group The group
+ * @returns Number of object types of the group 
+ * 
+ */
 int osync_group_num_objtypes(OSyncGroup *group)
 {
 	GList *objs = NULL;
@@ -889,6 +937,13 @@ int osync_group_num_objtypes(OSyncGroup *group)
 	return len;
 }
 
+/*! @brief The nth object type of the group
+ * 
+ * @param group The group
+ * @param nth The nth position of the object type list
+ * @returns Name of the nth object type of the group
+ * 
+ */
 const char *osync_group_nth_objtype(OSyncGroup *group, int nth)
 {
 	GList *objs = NULL;
@@ -901,6 +956,13 @@ const char *osync_group_nth_objtype(OSyncGroup *group, int nth)
 	
 }
 
+/*! @brief Change the status of an object type in the group.
+ * 
+ * @param group The group
+ * @param objtype Name of the object type
+ * @param enabled The status of the object type to set. TRUE enable, FALSE diable objtype. 
+ * 
+ */
 void osync_group_set_objtype_enabled(OSyncGroup *group, const char *objtype, osync_bool enabled)
 {
 	GList *m = NULL;
@@ -912,7 +974,13 @@ void osync_group_set_objtype_enabled(OSyncGroup *group, const char *objtype, osy
 	}
 }
 
-
+/*! @brief Get the status of an object type in the group 
+ * 
+ * @param group The group
+ * @param objtype The name of the object type 
+ * @returns TRUE if object type is enabled in this group, otherwise FALSE
+ * 
+ */
 int osync_group_objtype_enabled(OSyncGroup *group, const char *objtype)
 {
 	GList *m = NULL;
@@ -957,6 +1025,12 @@ int osync_group_objtype_enabled(OSyncGroup *group, const char *objtype)
 	return enabled;
 }
 
+/*! @brief Add filter to the group 
+ * 
+ * @param group The group
+ * @param filter The filter to add 
+ * 
+ */
 void osync_group_add_filter(OSyncGroup *group, OSyncFilter *filter)
 {
 	osync_assert(group);
@@ -964,6 +1038,12 @@ void osync_group_add_filter(OSyncGroup *group, OSyncFilter *filter)
 	osync_filter_ref(filter);
 }
 
+/*! @brief Remove filter from the group 
+ * 
+ * @param group The group
+ * @param filter The filter to remove
+ * 
+ */
 void osync_group_remove_filter(OSyncGroup *group, OSyncFilter *filter)
 {
 	osync_assert(group);
@@ -1029,6 +1109,13 @@ time_t osync_group_get_last_synchronization(OSyncGroup *group)
 	return group->last_sync;
 }
 
+/*! @brief Set fixed conflict resolution for the group for all appearing conflicts 
+ * 
+ * @param group The group
+ * @param res The conflict resolution
+ * @param num The Member ID which solves the conflict (winner)
+ * 
+ */
 void osync_group_set_conflict_resolution(OSyncGroup *group, OSyncConflictResolution res, int num)
 {
 	osync_assert(group);
@@ -1036,6 +1123,13 @@ void osync_group_set_conflict_resolution(OSyncGroup *group, OSyncConflictResolut
 	group->conflict_winner = num;
 }
 
+/*! @brief Get fixed conflict resolution for the group for all appearing conflicts 
+ * 
+ * @param group The group
+ * @param res Pointer to set conflict resolution value
+ * @param num Pointer to set Member ID value which solves the conflict (winner)
+ * 
+ */
 void osync_group_get_conflict_resolution(OSyncGroup *group, OSyncConflictResolution *res, int *num)
 {
 	osync_assert(group);
@@ -1046,4 +1140,52 @@ void osync_group_get_conflict_resolution(OSyncGroup *group, OSyncConflictResolut
 	*num = group->conflict_winner;
 }
 
+/*! @brief Get group configured status of merger use. 
+ * 
+ * @param group The group
+ * @return TRUE if merger is enabled. FALSE if merger is disabled.
+ */
+osync_bool osync_group_get_merger_enabled(OSyncGroup *group)
+{
+	osync_assert(group);
+	
+	return group->merger_enabled;
+}
+
+/*! @brief Configure status of merger use. 
+ * 
+ * @param group The group
+ * @param merger_enabled TRUE enables the merger. FALSE disables the merger. 
+ */
+void osync_group_set_merger_enabled(OSyncGroup *group, osync_bool merger_enabled)
+{
+	osync_assert(group);
+	
+	group->merger_enabled = merger_enabled;
+}
+
+/*! @brief Get group configured status of converter use. 
+ * 
+ * @param group The group
+ * @return TRUE if converter is enabled. FALSE if converter is disabled.
+ */
+osync_bool osync_group_get_converter_enabled(OSyncGroup *group)
+{
+	osync_assert(group);
+
+	return group->converter_enabled;
+}
+
+/*! @brief Configure status of converter use. 
+ * 
+ * @param group The group
+ * @param converter_enabled TRUE enables the converter. FALSE disables the converter. 
+ */
+void osync_group_set_converter_enabled(OSyncGroup *group, osync_bool converter_enabled)
+{
+	osync_assert(group);
+
+	group->converter_enabled = converter_enabled;
+}
+
 /*@}*/
