@@ -36,6 +36,63 @@
  */
 /*@{*/
 
+/** @brief Set Merger of Member 
+ * 
+ * @param member The Member pointer 
+ * 
+ */
+static void _osync_member_set_merger(OSyncMember *member, OSyncMerger *merger)
+{
+	osync_assert(member);
+	
+	if (member->merger)
+		osync_merger_unref(member->merger);
+	member->merger = merger;
+	if(merger)
+		osync_merger_ref(member->merger);
+}
+
+/** @brief Parse for the "objtype" node in  the member configuration
+ * 
+ * @param cur Pointer to the xmlNode 
+ * @param error Pointer to a error
+ * @returns Object type sink of the parsed configuration. NULL on error.
+ * 
+ */
+static OSyncObjTypeSink *_osync_member_parse_objtype(xmlNode *cur, OSyncError **error)
+{
+	osync_trace(TRACE_ENTRY, "%s(%p, %p)", __func__, cur, error);
+	OSyncObjTypeSink *sink = osync_objtype_sink_new(NULL, error);
+	if (!sink) {
+		osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(error));
+		return NULL;
+	}
+	
+	while (cur != NULL) {
+		char *str = (char*)xmlNodeGetContent(cur);
+		if (str) {
+			if (!xmlStrcmp(cur->name, (const xmlChar *)"name")) {
+				osync_objtype_sink_set_name(sink, str);
+			} else if (!xmlStrcmp(cur->name, (const xmlChar *)"enabled")) {
+				osync_objtype_sink_set_enabled(sink, atoi(str));
+			} else if (!xmlStrcmp(cur->name, (const xmlChar *)"read")) {
+				osync_objtype_sink_set_read(sink, atoi(str));
+			} else if (!xmlStrcmp(cur->name, (const xmlChar *)"getchanges")) {
+				osync_objtype_sink_set_getchanges(sink, atoi(str));
+			} else if (!xmlStrcmp(cur->name, (const xmlChar *)"write")) {
+				osync_objtype_sink_set_write(sink, atoi(str));
+			} else if (!xmlStrcmp(cur->name, (const xmlChar *)"objformat")) {
+				osync_objtype_sink_add_objformat(sink, str);
+			}
+			xmlFree(str);
+		}
+		cur = cur->next;
+	}
+	
+	osync_trace(TRACE_EXIT, "%s: %p", __func__, sink);
+	return sink;
+}
+
 /*@}*/
 
 /**
@@ -60,7 +117,7 @@ OSyncMember *osync_member_new(OSyncError **error)
 	member = osync_try_malloc0(sizeof(OSyncMember), error);
 	if (!member)
 		goto error;
-	member->starttype = OSYNC_START_TYPE_THREAD;
+
 	member->ref_count = 1;
 	
 	osync_trace(TRACE_EXIT, "%s: %p", __func__, member);
@@ -71,13 +128,25 @@ error:
 	return NULL;
 }
 
-void osync_member_ref(OSyncMember *member)
+/** @brief Increase the reference count of the member
+ * 
+ * @param member The member
+ * 
+ */
+OSyncMember *osync_member_ref(OSyncMember *member)
 {
 	osync_assert(member);
 	
 	g_atomic_int_inc(&(member->ref_count));
+
+	return member;
 }
 
+/** @brief Decrease the reference count of the member
+ * 
+ * @param member The member
+ * 
+ */
 void osync_member_unref(OSyncMember *member)
 {
 	osync_assert(member);
@@ -85,7 +154,10 @@ void osync_member_unref(OSyncMember *member)
 	if (g_atomic_int_dec_and_test(&(member->ref_count))) {
 		if (member->pluginname)
 			g_free(member->pluginname);
-		
+
+		if (member->name)
+			g_free(member->name);
+
 		if (member->configdir)
 			g_free(member->configdir);
 		
@@ -98,12 +170,8 @@ void osync_member_unref(OSyncMember *member)
 		if (osync_member_get_merger(member))
 			osync_merger_unref(osync_member_get_merger(member));
 		
-		while (member->objtypes) {
-			OSyncObjTypeSink *sink = member->objtypes->data;
-			osync_objtype_sink_unref(sink);
-			member->objtypes = g_list_remove(member->objtypes, member->objtypes->data);
-		}
-		
+		osync_member_flush_objtypes(member);
+
 		g_free(member);
 	}
 }
@@ -134,6 +202,32 @@ void osync_member_set_pluginname(OSyncMember *member, const char *pluginname)
 	member->pluginname = g_strdup(pluginname);
 }
 
+/** @brief Returns the inidividual name of the member
+ * 
+ * @param member The member
+ * @returns The name of the plugin
+ * 
+ */
+const char *osync_member_get_name(OSyncMember *member)
+{
+	osync_assert(member);
+	return member->name;
+}
+
+/** @brief Sets an individual name of the member
+ * 
+ * @param member The member
+ * @param name The individual name of the member 
+ * 
+ */
+void osync_member_set_name(OSyncMember *member, const char *name)
+{
+	osync_assert(member);
+	if (member->name)
+		g_free(member->name);
+	member->name = g_strdup(name);
+}
+
 /** @brief Returns the configuration directory where this member is stored
  * 
  * @param member The member
@@ -171,10 +265,8 @@ void osync_member_set_configdir(OSyncMember *member, const char *configdir)
  * directories
  * 
  * @param member The member
- * @param data Return location for the data
- * @param size Return location for the size of the data
  * @param error Pointer to a error
- * @returns TRUE if the config was loaded successfully, FALSE otherwise
+ * @returns The member configuration of the plugin default configuration if the member isn't configuered already 
  * 
  */
 const char *osync_member_get_config_or_default(OSyncMember *member, OSyncError **error)
@@ -247,10 +339,8 @@ osync_bool osync_member_has_config(OSyncMember *member)
  * configuration)
  * 
  * @param member The member
- * @param data Return location for the data
- * @param size Return location for the size of the data
  * @param error Pointer to a error
- * @returns TRUE if the config was loaded successfully, FALSE otherwise
+ * @returns Member configuration 
  * 
  */
 const char *osync_member_get_config(OSyncMember *member, OSyncError **error)
@@ -296,7 +386,6 @@ const char *osync_member_get_config(OSyncMember *member, OSyncError **error)
  * 
  * @param member The member
  * @param data The new config data
- * @param size The size of the data
  * 
  */
 void osync_member_set_config(OSyncMember *member, const char *data)
@@ -312,40 +401,12 @@ void osync_member_set_config(OSyncMember *member, const char *data)
 	osync_trace(TRACE_EXIT, "%s", __func__);
 }
 
-static OSyncObjTypeSink *_osync_member_parse_objtype(xmlNode *cur, OSyncError **error)
-{
-	osync_trace(TRACE_ENTRY, "%s(%p, %p)", __func__, cur, error);
-	OSyncObjTypeSink *sink = osync_objtype_sink_new(NULL, error);
-	if (!sink) {
-		osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(error));
-		return NULL;
-	}
-	
-	while (cur != NULL) {
-		char *str = (char*)xmlNodeGetContent(cur);
-		if (str) {
-			if (!xmlStrcmp(cur->name, (const xmlChar *)"name")) {
-				osync_objtype_sink_set_name(sink, str);
-			} else if (!xmlStrcmp(cur->name, (const xmlChar *)"enabled")) {
-				osync_objtype_sink_set_enabled(sink, atoi(str));
-			} else if (!xmlStrcmp(cur->name, (const xmlChar *)"objformat")) {
-				osync_objtype_sink_add_objformat(sink, str);
-			}
-			xmlFree(str);
-		}
-		cur = cur->next;
-	}
-	
-	osync_trace(TRACE_EXIT, "%s: %p", __func__, sink);
-	return sink;
-}
-
 /** @brief Loads a member from a directory where it has been saved
  * 
- * @param group The group which is the parent
+ * @param member The Member pointer of the member which gets loaded
  * @param path The path of the member
  * @param error Pointer to a error
- * @returns A newly allocated member thats stored in the group or NULL if error
+ * @returns TRUE on success, FALSE if error
  * 
  */
 osync_bool osync_member_load(OSyncMember *member, const char *path, OSyncError **error)
@@ -363,7 +424,7 @@ osync_bool osync_member_load(OSyncMember *member, const char *path, OSyncError *
 	g_free(basename);
 	osync_member_set_configdir(member, path);
 	
-	if (!osync_open_xml_file(&doc, &cur, filename, "syncmember", error)) {
+	if (!osync_xml_open_file(&doc, &cur, filename, "syncmember", error)) {
 		g_free(filename);
 		goto error;
 	}
@@ -378,6 +439,7 @@ osync_bool osync_member_load(OSyncMember *member, const char *path, OSyncError *
 				OSyncObjTypeSink *sink = _osync_member_parse_objtype(cur->xmlChildrenNode, error);
 				if (!sink)
 					goto error_free_doc;
+
 				member->objtypes = g_list_append(member->objtypes, sink);
 			}
 			xmlFree(str);
@@ -445,6 +507,9 @@ osync_bool osync_member_save(OSyncMember *member, OSyncError **error)
 		
 		xmlNewChild(node, NULL, (xmlChar*)"name", (xmlChar*)osync_objtype_sink_get_name(sink));
 		xmlNewChild(node, NULL, (xmlChar*)"enabled", osync_objtype_sink_is_enabled(sink) ? (xmlChar*)"1" : (xmlChar*)"0");
+		xmlNewChild(node, NULL, (xmlChar*)"read", osync_objtype_sink_get_read(sink) ? (xmlChar*)"1" : (xmlChar*)"0");
+		xmlNewChild(node, NULL, (xmlChar*)"getchanges", osync_objtype_sink_get_getchanges(sink) ? (xmlChar*)"1" : (xmlChar*)"0");
+		xmlNewChild(node, NULL, (xmlChar*)"write", osync_objtype_sink_get_write(sink) ? (xmlChar*)"1" : (xmlChar*)"0");
 		
 		int i = 0;
 		for (i = 0; i < osync_objtype_sink_num_objformats(sink); i++) {
@@ -453,7 +518,7 @@ osync_bool osync_member_save(OSyncMember *member, OSyncError **error)
 		}
 	}
 	
-	xmlSaveFile(filename, doc);
+	xmlSaveFormatFile(filename, doc, 1);
 	xmlFreeDoc(doc);
 	g_free(filename);
 	
@@ -486,6 +551,13 @@ error:
 	return FALSE;
 }
 
+/** @brief Delete a member
+ * 
+ * @param member The member to delete 
+ * @param error Pointer to a error
+ * @returns TRUE if the member was deleted successfully, FALSE otherwise
+ * 
+ */
 osync_bool osync_member_delete(OSyncMember *member, OSyncError **error)
 {
 	char *delcmd = NULL;
@@ -517,7 +589,15 @@ long long int osync_member_get_id(OSyncMember *member)
 	return member->id;
 }
 
-static OSyncObjTypeSink *_osync_member_find_objtype(OSyncMember *member, const char *objtype)
+/** @brief Find the object type sink (OSyncObjTypeSink) for the given object type of
+ *         a certain member.
+ * 
+ * @param member The member pointer
+ * @param objtype The searched object type 
+ * @returns OSyncObjTypeSink pointer if object type sink is avaliable, otherwise NULL 
+ * 
+ */
+OSyncObjTypeSink *osync_member_find_objtype_sink(OSyncMember *member, const char *objtype)
 {
 	GList *o;
 	for (o = member->objtypes; o; o = o->next) {
@@ -528,20 +608,35 @@ static OSyncObjTypeSink *_osync_member_find_objtype(OSyncMember *member, const c
 	return NULL;
 }
 
+/** @brief Add a specifc Object Format to member 
+ * 
+ * @param member The member pointer
+ * @param objtype The searched object type 
+ * @param objformat The name of the Object Format 
+ * 
+ */
 void osync_member_add_objformat(OSyncMember *member, const char *objtype, const char *format)
 {
-	OSyncObjTypeSink *sink = _osync_member_find_objtype(member, objtype);
+	OSyncObjTypeSink *sink = osync_member_find_objtype_sink(member, objtype);
 	if (!sink)
 		return;
 	
 	osync_objtype_sink_add_objformat(sink, format);
 }
 
+/** @brief List of all available object formats for a specifc object type of this member 
+ * 
+ * @param member The member pointer
+ * @param objtype The searched object type 
+ * @param error Pointer to a error
+ * @return List of all object formats of a specific object type of the member
+ * 
+ */
 const OSyncList *osync_member_get_objformats(OSyncMember *member, const char *objtype, OSyncError **error)
 {
-	OSyncObjTypeSink *sink = _osync_member_find_objtype(member, objtype);
+	OSyncObjTypeSink *sink = osync_member_find_objtype_sink(member, objtype);
 	if (!sink) {
-		sink = _osync_member_find_objtype(member, "data");
+		sink = osync_member_find_objtype_sink(member, "data");
 		if (!sink) {
 			osync_error_set(error, OSYNC_ERROR_GENERIC, "Unable to find objtype %s", objtype);
 			return NULL;
@@ -551,24 +646,55 @@ const OSyncList *osync_member_get_objformats(OSyncMember *member, const char *ob
 	return osync_objtype_sink_get_objformats(sink);
 }
 
-void osync_member_add_objtype(OSyncMember *member, const char *objtype)
+/** @brief Add an OSyncObjTypeSink object to the member list of supported object types of this member
+ * 
+ * @param member The member pointer
+ * @param sink The OSyncObjTypeSink object to add 
+ * 
+ */
+void osync_member_add_objtype_sink(OSyncMember *member, OSyncObjTypeSink *sink)
 {
-	OSyncObjTypeSink *sink = NULL;
 	osync_assert(member);
-	osync_assert(objtype);
+	osync_assert(sink);
 
-	if (!_osync_member_find_objtype(member, objtype)) {
-		sink = osync_objtype_sink_new(objtype, NULL);
-		member->objtypes = g_list_append(member->objtypes, sink);
-	}
+	member->objtypes = g_list_append(member->objtypes, sink);
+	osync_objtype_sink_ref(sink);
 }
 
+/** @brief Remove an OSyncObjTypeSink object to the member list of supported object types of this member
+ * 
+ * @param member The member pointer
+ * @param sink The OSyncObjTypeSink object to add 
+ * 
+ */
+void osync_member_remove_objtype_sink(OSyncMember *member, OSyncObjTypeSink *sink)
+{
+	osync_assert(member);
+	osync_assert(sink);
+
+	member->objtypes = g_list_remove(member->objtypes, sink);
+	osync_objtype_sink_unref(sink);
+}
+
+/** @brief The number of supported object types of this member
+ * 
+ * @param member The member pointer
+ * @returns Number of supported object type of this member
+ * 
+ */
 int osync_member_num_objtypes(OSyncMember *member)
 {
 	osync_assert(member);
 	return g_list_length(member->objtypes);
 }
 
+/** @brief The name of the nth supported object type of this member
+ * 
+ * @param member The member pointer
+ * @param nth The nth position of the list of supported object types of this member
+ * @returns Name of the nth supported object type
+ * 
+ */
 const char *osync_member_nth_objtype(OSyncMember *member, int nth)
 {
 	OSyncObjTypeSink *sink = NULL;
@@ -588,7 +714,7 @@ osync_bool osync_member_objtype_enabled(OSyncMember *member, const char *objtype
 {
 	OSyncObjTypeSink *sink = NULL;
 	osync_assert(member);
-	sink = _osync_member_find_objtype(member, objtype);
+	sink = osync_member_find_objtype_sink(member, objtype);
 	if (!sink)
 		return FALSE;
 	return osync_objtype_sink_is_enabled(sink);
@@ -612,7 +738,7 @@ void osync_member_set_objtype_enabled(OSyncMember *member, const char *objtype,
 	osync_trace(TRACE_ENTRY, "%s(%p, %s, %i)", __func__, member, objtype, enabled);
 	osync_assert(member);
 	
-	sink = _osync_member_find_objtype(member, objtype);
+	sink = osync_member_find_objtype_sink(member, objtype);
 	if (!sink) {
 		osync_trace(TRACE_EXIT, "%s: Unable to find objtype", __func__);
 		return;
@@ -622,24 +748,24 @@ void osync_member_set_objtype_enabled(OSyncMember *member, const char *objtype,
 	osync_trace(TRACE_EXIT, "%s", __func__);
 }
 
-void osync_member_set_start_type(OSyncMember *member, OSyncStartType type)
-{
-	osync_assert(member);
-	member->starttype = type;
-}
-
-OSyncStartType osync_member_get_start_type(OSyncMember *member)
-{
-	osync_assert(member);
-	return member->starttype;
-}
-
+/** @brief Get the capabilities of the member 
+ * 
+ * @param member The member
+ * @returns The capabilities of this member, NULL if no capabilities are set
+ */
 OSyncCapabilities *osync_member_get_capabilities(OSyncMember *member)
 {
 	osync_assert(member);
 	return member->capabilities;
 }
 
+/** @brief Set the capabilities of the member 
+ * 
+ * @param member The member
+ * @param capabilities The capabilities
+ * @param error Pointer to a error
+ * @returns TRUE if the capabilities got set successfully, otherwise FALSE 
+ */
 osync_bool osync_member_set_capabilities(OSyncMember *member, OSyncCapabilities *capabilities, OSyncError **error)
 {
 	osync_assert(member);
@@ -658,41 +784,34 @@ osync_bool osync_member_set_capabilities(OSyncMember *member, OSyncCapabilities
 	return TRUE;
 }
 
+/** @brief Get pointer of the Merger 
+ * 
+ * @param member The member
+ * @returns The pointer of the Merger, NULL if merger is disabled
+ */
 OSyncMerger *osync_member_get_merger(OSyncMember *member)
 {
 	osync_assert(member);
 	return member->merger;
 }
 
-void _osync_member_set_merger(OSyncMember *member, OSyncMerger *merger)
-{
-	osync_assert(member);
-	
-	if (member->merger)
-		osync_merger_unref(member->merger);
-	member->merger = merger;
-	if(merger)
-		osync_merger_ref(member->merger);
-}
-
-
 /** @brief Remove all object types from member. 
  * 
  * @param member The member
  *
  * Note: this function should be called to flush the member before discovering.
+ *       To detect if something isn't supported anymore.
  *
  */
 void osync_member_flush_objtypes(OSyncMember *member)
 {
 	osync_assert(member);
 
-	while (member->objtypes) {
-		OSyncObjTypeSink *sink = member->objtypes->data;
-		osync_objtype_sink_unref(sink);
-		member->objtypes = g_list_remove(member->objtypes, member->objtypes->data);
-	}
-
+        while (member->objtypes) {
+                OSyncObjTypeSink *sink = member->objtypes->data;
+                osync_objtype_sink_unref(sink);
+                member->objtypes = g_list_remove(member->objtypes, member->objtypes->data);
+        }
 }
 
 /*@}*/
