@@ -28,8 +28,9 @@ struct OSyncMember {
 	
 	char *configdata;
 	
-	OSyncGroup *group;
+	//OSyncGroup *group;
 
+	char *name;
 	char *pluginname;
 	
 	//For the filters
@@ -37,12 +38,8 @@ struct OSyncMember {
 	GList *filters;
 	int ref_count;
 	
-	OSyncStartType starttype;
-	
 	OSyncCapabilities *capabilities;
 	OSyncMerger *merger;
 };
 
-void _osync_member_set_merger(OSyncMember *member, OSyncMerger *merger);
-
 #endif /* _OPENSYNC_MEMBER_INTERNALS_H_ */
