@@ -1,6 +1,7 @@
 /*
  * libosengine - A synchronization engine for the opensync framework
  * Copyright (C) 2004-2005  Armin Bauer <armin.bauer@opensync.org>
+ * Copyright (C) 2007  Daniel Friedrich <daniel.friedrich@opensync.org>
  * 
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -66,7 +67,7 @@ gboolean _incoming_dispatch(GSource *source, GSourceFunc callback, gpointer user
 
 	while ((message = g_async_queue_try_pop(queue->incoming))) {
 		/* We check if the message is a reply to something */
-		osync_trace(TRACE_INTERNAL, "Dispatching %p:%i", message, osync_message_get_cmd(message));
+		osync_trace(TRACE_INTERNAL, "Dispatching %p:%i(%s)", message, osync_message_get_cmd(message), osync_message_get_commandstr(message));
 		
 		if (osync_message_get_cmd(message) == OSYNC_MESSAGE_REPLY || osync_message_get_cmd(message) == OSYNC_MESSAGE_ERRORREPLY) {
 			
@@ -140,7 +141,7 @@ gboolean _queue_check(GSource *source)
 	return FALSE;
 }
 
-int _osync_queue_write_data(OSyncQueue *queue, const void *vptr, size_t n, OSyncError **error)
+static int _osync_queue_write_data(OSyncQueue *queue, const void *vptr, size_t n, OSyncError **error)
 {
 	ssize_t nwritten = 0;
 
@@ -160,7 +161,7 @@ int _osync_queue_write_data(OSyncQueue *queue, const void *vptr, size_t n, OSync
 	return (nwritten);
 }
 
-osync_bool _osync_queue_write_long_long_int(OSyncQueue *queue, const long long int message, OSyncError **error)
+static osync_bool _osync_queue_write_long_long_int(OSyncQueue *queue, const long long int message, OSyncError **error)
 {
 	if (_osync_queue_write_data(queue, &message, sizeof(long long int), error) < 0)
 		return FALSE;
@@ -168,7 +169,7 @@ osync_bool _osync_queue_write_long_long_int(OSyncQueue *queue, const long long i
 	return TRUE;
 }
 
-osync_bool _osync_queue_write_int(OSyncQueue *queue, const int message, OSyncError **error)
+static osync_bool _osync_queue_write_int(OSyncQueue *queue, const int message, OSyncError **error)
 {
 	if (_osync_queue_write_data(queue, &message, sizeof(int), error) < 0)
 		return FALSE;
@@ -472,6 +473,8 @@ OSyncQueue *osync_queue_new(const char *name, OSyncError **error)
 	queue->outgoing = g_async_queue_new();
 	queue->incoming = g_async_queue_new();
 
+	queue->disconnectLock = g_mutex_new();
+
 	osync_trace(TRACE_EXIT, "%s: %p", __func__, queue);
 	return queue;
 
@@ -560,6 +563,8 @@ void osync_queue_free(OSyncQueue *queue)
 	
 	g_mutex_free(queue->pendingLock);
 	
+	g_mutex_free(queue->disconnectLock);
+
 	g_main_context_unref(queue->context);
 
 	_osync_queue_stop_incoming(queue);
@@ -584,6 +589,7 @@ void osync_queue_free(OSyncQueue *queue)
 		g_free(queue->name);
 		
 	g_free(queue);
+	queue = NULL;
 	
 	osync_trace(TRACE_EXIT, "%s", __func__);
 }
@@ -650,15 +656,16 @@ osync_bool osync_queue_connect(OSyncQueue *queue, OSyncQueueType type, OSyncErro
 			goto error;
 		}
 		queue->fd = fd;
-		int oldflags = fcntl(queue->fd, F_GETFD);
-		if (oldflags == -1) {
-			osync_error_set(error, OSYNC_ERROR_GENERIC, "Unable to get fifo flags");
-			goto error_close;
-		}
-		if (fcntl(queue->fd, F_SETFD, oldflags|FD_CLOEXEC) == -1) {
-			osync_error_set(error, OSYNC_ERROR_GENERIC, "Unable to set fifo flags");
-			goto error_close;
-		}
+	}
+
+	int oldflags = fcntl(queue->fd, F_GETFD);
+	if (oldflags == -1) {
+		osync_error_set(error, OSYNC_ERROR_GENERIC, "Unable to get fifo flags");
+		goto error_close;
+	}
+	if (fcntl(queue->fd, F_SETFD, oldflags|FD_CLOEXEC) == -1) {
+		osync_error_set(error, OSYNC_ERROR_GENERIC, "Unable to set fifo flags");
+		goto error_close;
 	}
 
 	queue->connected = TRUE;
@@ -715,7 +722,8 @@ osync_bool osync_queue_disconnect(OSyncQueue *queue, OSyncError **error)
 {
 	osync_trace(TRACE_ENTRY, "%s(%p, %p)", __func__, queue, error);
 	osync_assert(queue);
-	
+
+	g_mutex_lock(queue->disconnectLock);
 	if (queue->thread) {
 		osync_thread_stop(queue->thread);
 		osync_thread_free(queue->thread);
@@ -748,6 +756,7 @@ osync_bool osync_queue_disconnect(OSyncQueue *queue, OSyncError **error)
 	
 	queue->fd = -1;
 	queue->connected = FALSE;
+	g_mutex_unlock(queue->disconnectLock);
 	
 	osync_trace(TRACE_EXIT, "%s", __func__);
 	return TRUE;
@@ -837,10 +846,18 @@ OSyncQueueEvent osync_queue_poll(OSyncQueue *queue)
 	 * If we are sending, we can only receive a POLLERR which means that the remote side has
 	 * disconnected. Since we mainly dispatch the write IO, we dont want to block here. */
 	int ret = poll(&pfd, 1, queue->type == OSYNC_QUEUE_SENDER ? 0 : 100);
-	
-	if (ret == 0)
+
+	if (ret == 0) 
 		return OSYNC_QUEUE_EVENT_NONE;	
-	
+
+	/* Ignore interrupts. */
+	if  (ret < 0 && errno == EINTR) 
+		return OSYNC_QUEUE_EVENT_NONE;
+
+	if (ret < 0 )
+		osync_trace(TRACE_ERROR, "queue poll failed - system error :%i %s", errno, strerror(errno));
+
+
 	if (pfd.revents & POLLERR)
 		return OSYNC_QUEUE_EVENT_ERROR;
 	else if (pfd.revents & POLLHUP)
@@ -952,8 +969,30 @@ osync_bool osync_queue_is_alive(OSyncQueue *queue)
 	return TRUE;
 }
 
+/*! @brief Get the path of the fifo for the Queue
+ * 
+ * Get the full path of the fifo for this Queue if fifos used.
+ *
+ * @param queue The queue to get the file descriptor
+ * @returns The full path of the fifo or NULL if no fifos used 
+ * 
+ */
 const char *osync_queue_get_path(OSyncQueue *queue)
 {
 	osync_assert(queue);
 	return queue->name;
 }
+
+/*! @brief Get the pipe file descriptor of the Queue
+ * 
+ * Get the pipe file descriptor of this Queue if pipes are used.
+ *
+ * @param queue The queue to get the file descriptor
+ * @returns The pipe file descriptor of the queue or -1 if no pipes used or set
+ * 
+ */
+int osync_queue_get_fd(OSyncQueue *queue)
+{
+	osync_assert(queue);
+	return queue->fd;
+}
