@@ -42,6 +42,8 @@ struct OSyncQueue {
 	GSourceFuncs *read_functions;
 	GSource *read_source;
 	
+	GMutex *disconnectLock;
+
 	osync_bool connected;
 };
 
@@ -55,8 +57,4 @@ typedef struct OSyncPendingMessage {
 
 /*@}*/
 
-int _osync_queue_write_data(OSyncQueue *queue, const void *vptr, size_t n, OSyncError **error);
-osync_bool _osync_queue_write_long_long_int(OSyncQueue *queue, const long long int message, OSyncError **error);
-osync_bool _osync_queue_write_int(OSyncQueue *queue, const int message, OSyncError **error);
-
 #endif //_OPENSYNC_QUEUE_INTERNALS_H
