@@ -50,7 +50,9 @@ OSyncObjTypeSink *osync_objtype_sink_new(const char *objtype, OSyncError **error
 	sink->ref_count = 1;
 	
 	sink->read = TRUE;
+	sink->getchanges = TRUE;
 	sink->write = TRUE;
+
 	sink->enabled = TRUE;
 	
 	return sink;
@@ -61,11 +63,13 @@ OSyncObjTypeSink *osync_objtype_sink_new(const char *objtype, OSyncError **error
  * @param sink Pointer to the sink
  * 
  */
-void osync_objtype_sink_ref(OSyncObjTypeSink *sink)
+OSyncObjTypeSink *osync_objtype_sink_ref(OSyncObjTypeSink *sink)
 {
 	osync_assert(sink);
 	
 	g_atomic_int_inc(&(sink->ref_count));
+
+	return sink;
 }
 
 /*! @brief Decrease the reference count on a sink
@@ -216,6 +220,81 @@ void osync_objtype_sink_set_functions(OSyncObjTypeSink *sink, OSyncObjTypeSinkFu
 	osync_assert(sink);
 	sink->functions = functions;
 	sink->userdata = userdata;
+
+	if (functions.read)
+		 sink->func_read = TRUE;
+
+	if (functions.get_changes)
+		 sink->func_getchanges = TRUE;
+
+	if (functions.write)
+		 sink->func_write = TRUE;
+}
+
+/*! @brief Checks if sink has a read single entries function (read)
+ *
+ * @param sink Pointer to the sink
+ * @returns TRUE if the sink has a read single entries function (read), FALSE otherwise
+ */
+osync_bool osync_objtype_sink_get_function_read(OSyncObjTypeSink *sink)
+{
+	osync_assert(sink);
+	return sink->func_read;
+}
+
+/*! @brief Sets the status of the read sink function
+ *
+ * @param sink Pointer to sink
+ * @param write TRUE if the sink has a read function, FALSE otherwise
+ */
+void osync_objtype_sink_set_function_read(OSyncObjTypeSink *sink, osync_bool read)
+{
+	osync_assert(sink);
+	sink->func_read = read;
+}
+
+/*! @brief Checks if sink has a get latest changes function (get_changes)
+ *
+ * @param sink Pointer to the sink
+ * @returns TRUE if the sink has a get latest changes function (get_changes), FALSE otherwise
+ */
+osync_bool osync_objtype_sink_get_function_getchanges(OSyncObjTypeSink *sink)
+{
+	osync_assert(sink);
+	return sink->func_getchanges;
+}
+
+/*! @brief Sets the status of the get_changes sink function
+ *
+ * @param sink Pointer to sink
+ * @param write TRUE if the sink has a get_changes function, FALSE otherwise
+ */
+void osync_objtype_sink_set_function_getchanges(OSyncObjTypeSink *sink, osync_bool getchanges)
+{
+	osync_assert(sink);
+	sink->func_getchanges = getchanges;
+}
+
+/*! @brief Checks if sink has a write function (commit)
+ *
+ * @param sink Pointer to the sink
+ * @returns TRUE if the sink has a write function (commit), FALSE otherwise
+ */
+osync_bool osync_objtype_sink_get_function_write(OSyncObjTypeSink *sink)
+{
+	osync_assert(sink);
+	return sink->func_write;
+}
+
+/*! @brief Sets the status of the write sink function
+ *
+ * @param sink Pointer to sink
+ * @param write TRUE if the sink has a write function, FALSE otherwise
+ */
+void osync_objtype_sink_set_function_write(OSyncObjTypeSink *sink, osync_bool write)
+{
+	osync_assert(sink);
+	sink->func_write = write;
 }
 
 /** @brief Gets the user data from a sink
@@ -392,6 +471,11 @@ void osync_objtype_sink_commit_change(OSyncObjTypeSink *sink, void *plugindata,
 	if (functions.batch_commit) {
 		//Append to the stored changes
 		sink->commit_changes = g_list_append(sink->commit_changes, change);
+
+		/* Increment refcounting for batch_commit to avoid too early freeing of the context.
+		   Otherwise the context would get freed after this function call. But the batch_commit
+		   is collecting every contexts and changes and finally commits everything at once. */
+		osync_context_ref(ctx);
 		sink->commit_contexts = g_list_append(sink->commit_contexts, ctx);
 		osync_trace(TRACE_EXIT, "%s: Waiting for batch processing", __func__);
 		return;
@@ -514,24 +598,87 @@ void osync_objtype_sink_set_available(OSyncObjTypeSink *sink, osync_bool availab
 	sink->available = available;
 }
 
+/*! @brief Checks if sink is allowed to write (commit)
+ *
+ * If the sink is not allowed to write, then no changes will be commited to
+ * the sink.
+ *
+ * @param sink Pointer to the sink
+ * @returns TRUE if the sink is allowed to write (commit), FALSE otherwise
+ */
 osync_bool osync_objtype_sink_get_write(OSyncObjTypeSink *sink)
 {
 	osync_assert(sink);
 	return sink->write;
 }
 
+/*! @brief Sets the write status of the sink (commit)
+ *
+ * See osync_objtype_sink_get_write()
+ *
+ * @param sink Pointer to sink
+ * @param write TRUE if the sink is allowed to write changes (commit), FALSE otherwise
+ *
+ */
 void osync_objtype_sink_set_write(OSyncObjTypeSink *sink, osync_bool write)
 {
 	osync_assert(sink);
 	sink->write = write;
 }
 
+/*! @brief Sets the get latest changes status of the sink (get_change)
+ *
+ * See osync_objtype_sink_get_getchanges()
+ *
+ * @param sink Pointer to sink
+ * @param write Set TRUE if the sink is allowed to get latest changes, FALSE otherwise
+ *
+ */
+void osync_objtype_sink_set_getchanges(OSyncObjTypeSink *sink, osync_bool getchanges)
+{
+	osync_assert(sink);
+	sink->getchanges = getchanges;
+}
+
+/*! @brief Checks if sink is allowed to get latest changes 
+ *
+ * @param sink Pointer to the sink
+ * @returns TRUE if the sink is allowed to get latest changed entries, FALSE otherwise
+ *
+ */
+osync_bool osync_objtype_sink_get_getchanges(OSyncObjTypeSink *sink)
+{
+	osync_assert(sink);
+	return sink->getchanges;
+}
+
+/*! @brief Checks if sink is allowed to read single entries
+ *
+ * "Read" means to request a single entry and does not mean to get the
+ * latest changes since last sink. See osync_objtype_sink_get_getchanges().
+ * The read function explicitly means to read a single entry without triggering
+ * a full sync. This is used for example to check if a conflict between entries
+ * could be ignored. Ignoring conflicts is only possible if the sink is allowed to
+ * read this conflicting entries on the next sync without triggering a SlowSync.
+ *
+ * @param sink Pointer to the sink
+ * @returns TRUE if the sink is allowed to read single entries, FALSE otherwise
+ *
+ */
 osync_bool osync_objtype_sink_get_read(OSyncObjTypeSink *sink)
 {
 	osync_assert(sink);
 	return sink->read;
 }
 
+/*! @brief Sets the (single) read status of a sink 
+ *
+ * See osync_objtype_sink_get_read()
+ *
+ * @param sink Pointer to the sink
+ * @param read TRUE if the sink is able to read (single entries), FALSE otherwise
+ *
+ */
 void osync_objtype_sink_set_read(OSyncObjTypeSink *sink, osync_bool read)
 {
 	osync_assert(sink);
