@@ -10,247 +10,16 @@
 
 #include "../mock-plugin/mock_sync.h"
 
-
-
-int num_client_connected = 0;
-int num_client_main_connected = 0;
-int num_client_disconnected = 0;
-int num_client_main_disconnected = 0;
-int num_client_read = 0;
-int num_client_main_read = 0;
-int num_client_written = 0;
-int num_client_main_written = 0;
-int num_client_errors = 0;
-int num_client_sync_done = 0;
-int num_client_main_sync_done = 0;
-int num_client_discovered = 0;
-
-int num_change_read = 0;
-int num_change_written = 0;
-int num_change_error = 0;
-
-void member_status(OSyncMemberUpdate *status, void *user_data)
+static void _member_add_objtype(OSyncMember *member, const char *objtype)
 {
-	osync_trace(TRACE_ENTRY, "%s(%p (%i), %p)", __func__, status, status->type, user_data);
-	fail_unless(GINT_TO_POINTER(1) == user_data, NULL);
-	
-	switch (status->type) {
-		case OSYNC_CLIENT_EVENT_CONNECTED:
-			fail_unless(!osync_error_is_set(&(status->error)), NULL);
-			
-			if (status->objtype == NULL) {
-				num_client_main_connected++;
-			} else {
-				fail_unless(!strncmp(status->objtype, "mockobjtype", 11), NULL);
-				num_client_connected++;
-			}
-			
-			break;
-		case OSYNC_CLIENT_EVENT_DISCONNECTED:
-			fail_unless(!osync_error_is_set(&(status->error)), NULL);
-			
-			if (status->objtype == NULL) {
-				num_client_main_disconnected++;
-			} else {
-				fail_unless(!strncmp(status->objtype, "mockobjtype", 11), NULL);
-				num_client_disconnected++;
-			}
-			
-			break;
-		case OSYNC_CLIENT_EVENT_READ:
-			fail_unless(!osync_error_is_set(&(status->error)), NULL);
-			
-			if (status->objtype == NULL) {
-				num_client_main_read++;
-			} else {
-				fail_unless(!strncmp(status->objtype, "mockobjtype", 11), NULL);
-				num_client_read++;
-			}
-			
-			break;
-		case OSYNC_CLIENT_EVENT_WRITTEN:
-			fail_unless(!osync_error_is_set(&(status->error)), NULL);
-			
-			if (status->objtype == NULL) {
-				num_client_main_written++;
-			} else {
-				fail_unless(!strncmp(status->objtype, "mockobjtype", 11), NULL);
-				num_client_written++;
-			}
-			
-			break;
-		case OSYNC_CLIENT_EVENT_ERROR:
-			fail_unless(osync_error_is_set(&(status->error)), NULL);
-			num_client_errors++;
-			break;
-		case OSYNC_CLIENT_EVENT_SYNC_DONE:
-			fail_unless(!osync_error_is_set(&(status->error)), NULL);
-			
-			if (status->objtype == NULL) {
-				num_client_main_sync_done++;
-			} else {
-				fail_unless(!strncmp(status->objtype, "mockobjtype", 11), NULL);
-				num_client_sync_done++;
-			}
-			
-			break;
-		case OSYNC_CLIENT_EVENT_DISCOVERED:
-			fail_unless(!osync_error_is_set(&(status->error)), NULL);
-			num_client_discovered++;
-			break;
-	}
-	
-	osync_trace(TRACE_EXIT, "%s", __func__);
-}
+       OSyncObjTypeSink *sink = NULL;
+       osync_assert(member);
+       osync_assert(objtype);
 
-void entry_status(OSyncChangeUpdate *status, void *user_data)
-{
-	osync_trace(TRACE_ENTRY, "%s(%p (%i), %p)", __func__, status, status->type, user_data);
-	
-	switch (status->type) {
-		case OSYNC_CHANGE_EVENT_READ:
-			fail_unless(!osync_error_is_set(&(status->error)), NULL);
-			num_change_read++;
-			break;
-		case OSYNC_CHANGE_EVENT_WRITTEN:
-			fail_unless(!osync_error_is_set(&(status->error)), NULL);
-			num_change_written++;
-			break;
-		case OSYNC_CHANGE_EVENT_ERROR:
-			fail_unless(osync_error_is_set(&(status->error)), NULL);
-			num_change_error++;
-			break;
-	}
-	
-	osync_trace(TRACE_EXIT, "%s", __func__);
-}
-
-int num_engine_connected = 0;
-int num_engine_read = 0;
-int num_engine_written = 0;
-int num_engine_disconnected = 0;
-int num_engine_errors = 0;
-int num_engine_successful = 0;
-int num_engine_end_conflicts = 0;
-int num_engine_prev_unclean = 0;
-int num_engine_sync_done = 0;
-
-void engine_status(OSyncEngineUpdate *status, void *user_data)
-{
-	switch (status->type) {
-		case OSYNC_ENGINE_EVENT_CONNECTED:
-			fail_unless(!osync_error_is_set(&(status->error)), NULL);
-			num_engine_connected++;
-			break;
-		case OSYNC_ENGINE_EVENT_READ:
-			fail_unless(!osync_error_is_set(&(status->error)), NULL);
-			num_engine_read++;
-			break;
-		case OSYNC_ENGINE_EVENT_WRITTEN:
-			fail_unless(!osync_error_is_set(&(status->error)), NULL);
-			num_engine_written++;
-			break;
-		case OSYNC_ENGINE_EVENT_DISCONNECTED:
-			fail_unless(!osync_error_is_set(&(status->error)), NULL);
-			num_engine_disconnected++;
-			break;
-		case OSYNC_ENGINE_EVENT_ERROR:
-			fail_unless(osync_error_is_set(&(status->error)), NULL);
-			num_engine_errors++;
-			break;
-		case OSYNC_ENGINE_EVENT_SUCCESSFUL:
-			fail_unless(!osync_error_is_set(&(status->error)), NULL);
-			num_engine_successful++;
-			break;
-		case OSYNC_ENGINE_EVENT_PREV_UNCLEAN:
-			fail_unless(!osync_error_is_set(&(status->error)), NULL);
-			num_engine_prev_unclean++;
-			break;
-		case OSYNC_ENGINE_EVENT_END_CONFLICTS:
-			fail_unless(!osync_error_is_set(&(status->error)), NULL);
-			num_engine_end_conflicts++;
-			break;
-		case OSYNC_ENGINE_EVENT_SYNC_DONE:
-			fail_unless(!osync_error_is_set(&(status->error)), NULL);
-			num_engine_sync_done++;
-			break;
-	}
-}
-
-int num_mapping_solved = 0;
-int num_mapping_written = 0;
-int num_mapping_errors = 0;
-
-void mapping_status(OSyncMappingUpdate *status, void *user_data)
-{
-	switch (status->type) {
-		case OSYNC_MAPPING_EVENT_SOLVED:
-			fail_unless(!osync_error_is_set(&(status->error)), NULL);
-			num_mapping_solved++;
-			break;
-		/*case OSYNC_MAPPING_EVENT_WRITTEN:
-			fail_unless(!osync_error_is_set(&(status->error)), NULL);
-			num_mapping_written++;
-			break;*/
-		case OSYNC_MAPPING_EVENT_ERROR:
-			fail_unless(osync_error_is_set(&(status->error)), NULL);
-			num_mapping_errors++;
-			break;
-	}
-}
-
-int num_mapping_conflicts = 0;
-
-static void conflict_handler_choose_first(OSyncEngine *engine, OSyncMappingEngine *mapping, void *user_data)
-{
-	osync_trace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, engine, mapping, user_data);
-	
-	num_mapping_conflicts++;
-	fail_unless(osync_mapping_engine_num_changes(mapping) == GPOINTER_TO_INT(user_data), NULL);
-	fail_unless(num_engine_end_conflicts == 0, NULL);
-
-	OSyncChange *change = osync_mapping_engine_member_change(mapping, 1);
-	OSyncError *error = NULL;
-	osync_assert(osync_engine_mapping_solve(engine, mapping, change, &error));
-	osync_assert(error == NULL);
-	
-	osync_trace(TRACE_EXIT, "%s", __func__);
-}
-
-static void reset_counters()
-{
-	num_client_connected = 0;
-	num_client_main_connected = 0;
-	num_client_disconnected = 0;
-	num_client_main_disconnected = 0;
-	num_client_read = 0;
-	num_client_main_read = 0;
-	num_client_written = 0;
-	num_client_main_written = 0;
-	num_client_errors = 0;
-	num_client_sync_done = 0;
-	num_client_main_sync_done = 0;
-	num_client_discovered = 0;
-	
-	num_change_read = 0;
-	num_change_written = 0;
-	num_change_error = 0;
-	
-	num_engine_connected = 0;
-	num_engine_read = 0;
-	num_engine_written = 0;
-	num_engine_disconnected = 0;
-	num_engine_errors = 0;
-	num_engine_successful = 0;
-	num_engine_end_conflicts = 0;
-	num_engine_prev_unclean = 0;
-	num_engine_sync_done = 0;
-	
-	num_mapping_solved = 0;
-	num_mapping_written = 0;
-	num_mapping_errors = 0;
-	
-	num_mapping_conflicts = 0;
+       if (!osync_member_find_objtype_sink(member, objtype)) {
+               sink = osync_objtype_sink_new(objtype, NULL);
+	       osync_member_add_objtype_sink(member, sink);
+       }
 }
 
 int num_connect = 0;
@@ -398,12 +167,11 @@ static OSyncDebugGroup *_create_group(char *testbed)
 	fail_unless(debug->member1 != NULL, NULL);
 	fail_unless(error == NULL, NULL);
 	osync_group_add_member(debug->group, debug->member1);
-	osync_member_set_pluginname(debug->member1, "mock-sync");
+	osync_member_set_pluginname(debug->member1, "mock-sync-external");
 	path = g_strdup_printf("%s/configs/group/1", testbed);
 	osync_member_set_configdir(debug->member1, path);
 	g_free(path);
-	osync_member_set_start_type(debug->member1, OSYNC_START_TYPE_EXTERNAL);
-	osync_member_add_objtype(debug->member1, "mockobjtype1");
+	_member_add_objtype(debug->member1, "mockobjtype1");
 	osync_member_add_objformat(debug->member1, "mockobjtype1", "mockformat1");
 	osync_member_set_config(debug->member1, "<config><directory><path>data1</path><objtype>mockobjtype1</objtype></directory></config>");
 	
@@ -411,12 +179,12 @@ static OSyncDebugGroup *_create_group(char *testbed)
 	fail_unless(debug->member2 != NULL, NULL);
 	fail_unless(error == NULL, NULL);
 	osync_group_add_member(debug->group, debug->member2);
-	osync_member_set_pluginname(debug->member2, "mock-sync2");
+	osync_member_set_pluginname(debug->member2, "mock-sync-external");
 	path = g_strdup_printf("%s/configs/group/2", testbed);
 	osync_member_set_configdir(debug->member2, path);
 	g_free(path);
-	osync_member_set_start_type(debug->member2, OSYNC_START_TYPE_EXTERNAL);
-	osync_member_add_objtype(debug->member2, "mockobjtype1");
+
+	_member_add_objtype(debug->member2, "mockobjtype1");
 	osync_member_add_objformat(debug->member2, "mockobjtype1", "mockformat1");
 	osync_member_set_config(debug->member2, "<config><directory><path>data2</path><objtype>mockobjtype1</objtype></directory></config>");
 	
@@ -426,9 +194,10 @@ static OSyncDebugGroup *_create_group(char *testbed)
 	fail_unless(debug->plugin != NULL, NULL);
 	fail_unless(error == NULL, NULL);
 	
-	osync_plugin_set_name(debug->plugin, "mock-sync");
+	osync_plugin_set_name(debug->plugin, "mock-sync-external");
 	osync_plugin_set_longname(debug->plugin, "Mock Sync Plugin");
 	osync_plugin_set_description(debug->plugin, "This is a pseudo plugin");
+	osync_plugin_set_start_type(debug->plugin, OSYNC_START_TYPE_EXTERNAL);
 	
 	osync_plugin_set_initialize(debug->plugin, initialize);
 	osync_plugin_set_finalize(debug->plugin, finalize);
@@ -438,9 +207,10 @@ static OSyncDebugGroup *_create_group(char *testbed)
 	fail_unless(debug->plugin2 != NULL, NULL);
 	fail_unless(error == NULL, NULL);
 	
-	osync_plugin_set_name(debug->plugin2, "mock-sync2");
+	osync_plugin_set_name(debug->plugin2, "mock-sync-external");
 	osync_plugin_set_longname(debug->plugin2, "Mock Sync Plugin");
 	osync_plugin_set_description(debug->plugin2, "This is a pseudo plugin");
+	osync_plugin_set_start_type(debug->plugin2, OSYNC_START_TYPE_EXTERNAL);
 	
 	osync_plugin_set_initialize(debug->plugin2, initialize_error);
 	osync_plugin_set_finalize(debug->plugin2, finalize);
@@ -481,12 +251,12 @@ static OSyncDebugGroup *_create_group2(char *testbed)
 	fail_unless(debug->member1 != NULL, NULL);
 	fail_unless(error == NULL, NULL);
 	osync_group_add_member(debug->group, debug->member1);
-	osync_member_set_pluginname(debug->member1, "mock-sync");
+	osync_member_set_pluginname(debug->member1, "mock-sync-external");
 	path = g_strdup_printf("%s/configs/group/1", testbed);
 	osync_member_set_configdir(debug->member1, path);
 	g_free(path);
-	osync_member_set_start_type(debug->member1, OSYNC_START_TYPE_EXTERNAL);
-	osync_member_add_objtype(debug->member1, "mockobjtype1");
+
+	_member_add_objtype(debug->member1, "mockobjtype1");
 	osync_member_add_objformat(debug->member1, "mockobjtype1", "mockformat1");
 	osync_member_set_config(debug->member1, "<config><directory><path>data1</path><objtype>mockobjtype1</objtype></directory></config>");
 	
@@ -494,12 +264,12 @@ static OSyncDebugGroup *_create_group2(char *testbed)
 	fail_unless(debug->member2 != NULL, NULL);
 	fail_unless(error == NULL, NULL);
 	osync_group_add_member(debug->group, debug->member2);
-	osync_member_set_pluginname(debug->member2, "mock-sync2");
+	osync_member_set_pluginname(debug->member2, "mock-sync-external");
 	path = g_strdup_printf("%s/configs/group/2", testbed);
 	osync_member_set_configdir(debug->member2, path);
 	g_free(path);
-	osync_member_set_start_type(debug->member2, OSYNC_START_TYPE_EXTERNAL);
-	osync_member_add_objtype(debug->member2, "mockobjtype1");
+
+	_member_add_objtype(debug->member2, "mockobjtype1");
 	osync_member_add_objformat(debug->member2, "mockobjtype1", "mockformat1");
 	osync_member_set_config(debug->member2, "<config><directory><path>data2</path><objtype>mockobjtype1</objtype></directory></config>");
 	
@@ -509,9 +279,10 @@ static OSyncDebugGroup *_create_group2(char *testbed)
 	fail_unless(debug->plugin != NULL, NULL);
 	fail_unless(error == NULL, NULL);
 	
-	osync_plugin_set_name(debug->plugin, "mock-sync");
+	osync_plugin_set_name(debug->plugin, "mock-sync-external");
 	osync_plugin_set_longname(debug->plugin, "Mock Sync Plugin");
 	osync_plugin_set_description(debug->plugin, "This is a pseudo plugin");
+	osync_plugin_set_start_type(debug->plugin, OSYNC_START_TYPE_EXTERNAL);
 	
 	osync_plugin_set_initialize(debug->plugin, initialize_error);
 	osync_plugin_set_finalize(debug->plugin, finalize);
@@ -521,9 +292,10 @@ static OSyncDebugGroup *_create_group2(char *testbed)
 	fail_unless(debug->plugin2 != NULL, NULL);
 	fail_unless(error == NULL, NULL);
 	
-	osync_plugin_set_name(debug->plugin2, "mock-sync2");
+	osync_plugin_set_name(debug->plugin2, "mock-sync-external");
 	osync_plugin_set_longname(debug->plugin2, "Mock Sync Plugin");
 	osync_plugin_set_description(debug->plugin2, "This is a pseudo plugin");
+	osync_plugin_set_start_type(debug->plugin2, OSYNC_START_TYPE_EXTERNAL);
 	
 	osync_plugin_set_initialize(debug->plugin2, initialize_error);
 	osync_plugin_set_finalize(debug->plugin2, finalize);
@@ -564,24 +336,24 @@ static OSyncDebugGroup *_create_group3(char *testbed)
 	fail_unless(debug->member1 != NULL, NULL);
 	fail_unless(error == NULL, NULL);
 	osync_group_add_member(debug->group, debug->member1);
-	osync_member_set_pluginname(debug->member1, "mock-sync");
+	osync_member_set_pluginname(debug->member1, "mock-sync-external");
 	path = g_strdup_printf("%s/configs/group/1", testbed);
 	osync_member_set_configdir(debug->member1, path);
 	g_free(path);
-	osync_member_set_start_type(debug->member1, OSYNC_START_TYPE_EXTERNAL);
-	osync_member_add_objtype(debug->member1, "mockobjtype1");
+
+	_member_add_objtype(debug->member1, "mockobjtype1");
 	osync_member_add_objformat(debug->member1, "mockobjtype1", "mockformat1");
 	
 	debug->member2 = osync_member_new(&error);
 	fail_unless(debug->member2 != NULL, NULL);
 	fail_unless(error == NULL, NULL);
 	osync_group_add_member(debug->group, debug->member2);
-	osync_member_set_pluginname(debug->member2, "mock-sync2");
+	osync_member_set_pluginname(debug->member2, "mock-sync-external");
 	path = g_strdup_printf("%s/configs/group/2", testbed);
 	osync_member_set_configdir(debug->member2, path);
 	g_free(path);
-	osync_member_set_start_type(debug->member2, OSYNC_START_TYPE_EXTERNAL);
-	osync_member_add_objtype(debug->member2, "mockobjtype1");
+
+	_member_add_objtype(debug->member2, "mockobjtype1");
 	osync_member_add_objformat(debug->member2, "mockobjtype1", "mockformat1");
 	
 	
@@ -590,9 +362,10 @@ static OSyncDebugGroup *_create_group3(char *testbed)
 	fail_unless(debug->plugin != NULL, NULL);
 	fail_unless(error == NULL, NULL);
 	
-	osync_plugin_set_name(debug->plugin, "mock-sync");
+	osync_plugin_set_name(debug->plugin, "mock-sync-external");
 	osync_plugin_set_longname(debug->plugin, "Mock Sync Plugin");
 	osync_plugin_set_description(debug->plugin, "This is a pseudo plugin");
+	osync_plugin_set_start_type(debug->plugin, OSYNC_START_TYPE_EXTERNAL);
 	
 	osync_plugin_set_initialize(debug->plugin, initialize_error);
 	osync_plugin_set_finalize(debug->plugin, finalize);
@@ -602,9 +375,10 @@ static OSyncDebugGroup *_create_group3(char *testbed)
 	fail_unless(debug->plugin2 != NULL, NULL);
 	fail_unless(error == NULL, NULL);
 	
-	osync_plugin_set_name(debug->plugin2, "mock-sync2");
+	osync_plugin_set_name(debug->plugin2, "mock-sync-external");
 	osync_plugin_set_longname(debug->plugin2, "Mock Sync Plugin");
 	osync_plugin_set_description(debug->plugin2, "This is a pseudo plugin");
+	osync_plugin_set_start_type(debug->plugin2, OSYNC_START_TYPE_EXTERNAL);
 	
 	osync_plugin_set_initialize(debug->plugin2, initialize_error);
 	osync_plugin_set_finalize(debug->plugin2, finalize);
@@ -645,12 +419,12 @@ static OSyncDebugGroup *_create_group4(char *testbed)
 	fail_unless(debug->member1 != NULL, NULL);
 	fail_unless(error == NULL, NULL);
 	osync_group_add_member(debug->group, debug->member1);
-	osync_member_set_pluginname(debug->member1, "mock-sync");
+	osync_member_set_pluginname(debug->member1, "mock-sync-external");
 	path = g_strdup_printf("%s/configs/group/1", testbed);
 	osync_member_set_configdir(debug->member1, path);
 	g_free(path);
-	osync_member_set_start_type(debug->member1, OSYNC_START_TYPE_EXTERNAL);
-	osync_member_add_objtype(debug->member1, "mockobjtype1");
+
+	_member_add_objtype(debug->member1, "mockobjtype1");
 	osync_member_add_objformat(debug->member1, "mockobjtype1", "mockformat1");
 	osync_member_set_config(debug->member1, "<config><directory><path>data1</path><objtype>mockobjtype1</objtype></directory></config>");
 	
@@ -658,11 +432,11 @@ static OSyncDebugGroup *_create_group4(char *testbed)
 	fail_unless(debug->member2 != NULL, NULL);
 	fail_unless(error == NULL, NULL);
 	osync_group_add_member(debug->group, debug->member2);
-	osync_member_set_pluginname(debug->member2, "mock-sync2");
+	osync_member_set_pluginname(debug->member2, "mock-sync-external");
 	path = g_strdup_printf("%s/configs/group/2", testbed);
 	osync_member_set_configdir(debug->member2, path);
 	g_free(path);
-	osync_member_set_start_type(debug->member2, OSYNC_START_TYPE_EXTERNAL);
+
 	osync_member_set_config(debug->member2, "<config><directory><path>data2</path><objtype>mockobjtype1</objtype></directory></config>");
 	
 	
@@ -671,9 +445,10 @@ static OSyncDebugGroup *_create_group4(char *testbed)
 	fail_unless(debug->plugin != NULL, NULL);
 	fail_unless(error == NULL, NULL);
 	
-	osync_plugin_set_name(debug->plugin, "mock-sync");
+	osync_plugin_set_name(debug->plugin, "mock-sync-external");
 	osync_plugin_set_longname(debug->plugin, "Mock Sync Plugin");
 	osync_plugin_set_description(debug->plugin, "This is a pseudo plugin");
+	osync_plugin_set_start_type(debug->plugin, OSYNC_START_TYPE_EXTERNAL);
 	
 	osync_plugin_set_initialize(debug->plugin, initialize);
 	osync_plugin_set_finalize(debug->plugin, finalize);
@@ -683,9 +458,10 @@ static OSyncDebugGroup *_create_group4(char *testbed)
 	fail_unless(debug->plugin2 != NULL, NULL);
 	fail_unless(error == NULL, NULL);
 	
-	osync_plugin_set_name(debug->plugin2, "mock-sync2");
+	osync_plugin_set_name(debug->plugin2, "mock-sync-external");
 	osync_plugin_set_longname(debug->plugin2, "Mock Sync Plugin");
 	osync_plugin_set_description(debug->plugin2, "This is a pseudo plugin");
+	osync_plugin_set_start_type(debug->plugin2, OSYNC_START_TYPE_EXTERNAL);
 	
 	osync_plugin_set_initialize(debug->plugin2, initialize);
 	osync_plugin_set_finalize(debug->plugin2, finalize);
@@ -726,12 +502,12 @@ static OSyncDebugGroup *_create_group5(char *testbed)
 	fail_unless(debug->member1 != NULL, NULL);
 	fail_unless(error == NULL, NULL);
 	osync_group_add_member(debug->group, debug->member1);
-	osync_member_set_pluginname(debug->member1, "mock-sync");
+	osync_member_set_pluginname(debug->member1, "mock-sync-external");
 	path = g_strdup_printf("%s/configs/group/1", testbed);
 	osync_member_set_configdir(debug->member1, path);
 	g_free(path);
-	osync_member_set_start_type(debug->member1, OSYNC_START_TYPE_EXTERNAL);
-	osync_member_add_objtype(debug->member1, "mockobjtype1");
+
+	_member_add_objtype(debug->member1, "mockobjtype1");
 	osync_member_add_objformat(debug->member1, "mockobjtype1", "mockformat1");
 	osync_member_set_config(debug->member1, "<config><directory><path>data1</path><objtype>mockobjtype1</objtype></directory></config>");
 	
@@ -739,24 +515,23 @@ static OSyncDebugGroup *_create_group5(char *testbed)
 	fail_unless(debug->member2 != NULL, NULL);
 	fail_unless(error == NULL, NULL);
 	osync_group_add_member(debug->group, debug->member2);
-	osync_member_set_pluginname(debug->member2, "mock-sync2");
+	osync_member_set_pluginname(debug->member2, "mock-sync-external");
 	path = g_strdup_printf("%s/configs/group/2", testbed);
 	osync_member_set_configdir(debug->member2, path);
 	g_free(path);
-	osync_member_set_start_type(debug->member2, OSYNC_START_TYPE_EXTERNAL);
-	osync_member_add_objtype(debug->member2, "mockobjtype1");
+	_member_add_objtype(debug->member2, "mockobjtype1");
 	osync_member_add_objformat(debug->member2, "mockobjtype1", "mockformat1");
 	osync_member_set_config(debug->member2, "<config><directory><path>data2</path><objtype>mockobjtype1</objtype></directory></config>");
 	
 	
-	
 	debug->plugin = osync_plugin_new(&error);
 	fail_unless(debug->plugin != NULL, NULL);
 	fail_unless(error == NULL, NULL);
 	
-	osync_plugin_set_name(debug->plugin, "mock-sync");
+	osync_plugin_set_name(debug->plugin, "mock-sync-external");
 	osync_plugin_set_longname(debug->plugin, "Mock Sync Plugin");
 	osync_plugin_set_description(debug->plugin, "This is a pseudo plugin");
+	osync_plugin_set_start_type(debug->plugin, OSYNC_START_TYPE_EXTERNAL);
 	
 	osync_plugin_set_initialize(debug->plugin, initialize_connect_error);
 	osync_plugin_set_finalize(debug->plugin, finalize);
@@ -766,16 +541,15 @@ static OSyncDebugGroup *_create_group5(char *testbed)
 	fail_unless(debug->plugin2 != NULL, NULL);
 	fail_unless(error == NULL, NULL);
 	
-	osync_plugin_set_name(debug->plugin2, "mock-sync2");
+	osync_plugin_set_name(debug->plugin2, "mock-sync-external");
 	osync_plugin_set_longname(debug->plugin2, "Mock Sync Plugin");
 	osync_plugin_set_description(debug->plugin2, "This is a pseudo plugin");
+	osync_plugin_set_start_type(debug->plugin2, OSYNC_START_TYPE_EXTERNAL);
 	
 	osync_plugin_set_initialize(debug->plugin2, initialize_connect_error);
 	osync_plugin_set_finalize(debug->plugin2, finalize);
 	
 	
-	
-	
 	debug->client1 = osync_client_new(&error);
 	fail_unless(debug->client1 != NULL, NULL);
 	fail_unless(error == NULL, NULL);
@@ -819,6 +593,7 @@ START_TEST (single_init_error)
 	fail_unless(engine != NULL, NULL);
 	fail_unless(error == NULL, NULL);
 	osync_engine_set_plugindir(engine, testbed);
+	osync_engine_set_formatdir(engine, testbed);
 	
 	fail_unless(!osync_engine_initialize(engine, &error), NULL);
 	fail_unless(error != NULL, NULL);
@@ -847,6 +622,7 @@ START_TEST (double_init_error)
 	fail_unless(engine != NULL, NULL);
 	fail_unless(error == NULL, NULL);
 	osync_engine_set_plugindir(engine, testbed);
+	osync_engine_set_formatdir(engine, testbed);
 	
 	fail_unless(!osync_engine_initialize(engine, &error), NULL);
 	fail_unless(error != NULL, NULL);
@@ -875,6 +651,7 @@ START_TEST (no_config_error)
 	fail_unless(engine != NULL, NULL);
 	fail_unless(error == NULL, NULL);
 	osync_engine_set_plugindir(engine, testbed);
+	osync_engine_set_formatdir(engine, testbed);
 	
 	fail_unless(!osync_engine_initialize(engine, &error), NULL);
 	fail_unless(error != NULL, NULL);
@@ -903,6 +680,7 @@ START_TEST (no_objtype_error)
 	fail_unless(engine != NULL, NULL);
 	fail_unless(error == NULL, NULL);
 	osync_engine_set_plugindir(engine, testbed);
+	osync_engine_set_formatdir(engine, testbed);
 	
 	fail_unless(!osync_engine_initialize(engine, &error), NULL);
 	fail_unless(error != NULL, NULL);
@@ -931,6 +709,7 @@ START_TEST (dual_connect_error)
 	fail_unless(engine != NULL, NULL);
 	fail_unless(error == NULL, NULL);
 	osync_engine_set_plugindir(engine, testbed);
+	osync_engine_set_formatdir(engine, testbed);
 	
 	osync_engine_set_conflict_callback(engine, conflict_handler_choose_first, GINT_TO_POINTER(1));
 	osync_engine_set_changestatus_callback(engine, entry_status, GINT_TO_POINTER(1));
@@ -940,7 +719,7 @@ START_TEST (dual_connect_error)
 	
 	fail_unless(osync_engine_initialize(engine, &error), NULL);
 	fail_unless(error == NULL, NULL);
-	
+
 	fail_unless(!osync_engine_synchronize_and_block(engine, &error), NULL);
 	fail_unless(error != NULL, NULL);
 	osync_error_unref(&error);
@@ -2841,7 +2620,7 @@ int main(void)
 	
 	SRunner *sr;
 	sr = srunner_create(s);
-	srunner_run_all(sr, CK_NORMAL);
+	srunner_run_all(sr, CK_VERBOSE);
 	nf = srunner_ntests_failed(sr);
 	srunner_free(sr);
 	return (nf == 0) ? EXIT_SUCCESS : EXIT_FAILURE;
