@@ -638,6 +638,7 @@ static osync_bool osync_filesync_discover(void *data, OSyncPluginInfo *info, OSy
 	
 	OSyncVersion *version = osync_version_new(error);
 	osync_version_set_plugin(version, "mock-sync");
+	//osync_version_set_vendor(version, "version");
 	//osync_version_set_modelversion(version, "version");
 	//osync_version_set_firmwareversion(version, "firmwareversion");
 	//osync_version_set_softwareversion(version, "softwareversion");
@@ -668,6 +669,23 @@ osync_bool get_sync_info(OSyncPluginEnv *env, OSyncError **error)
 	
 	osync_plugin_env_register_plugin(env, plugin);
 	osync_plugin_unref(plugin);
+
+	plugin = osync_plugin_new(error);
+	if (!plugin)
+		goto error;
+	
+	osync_plugin_set_name(plugin, "mock-sync-external");
+	osync_plugin_set_longname(plugin, "Mock Synchronization Plugin with Start Type External");
+	osync_plugin_set_description(plugin, "Plugin to synchronize files on the local filesystem for unit tests");
+	osync_plugin_set_start_type(plugin, OSYNC_START_TYPE_EXTERNAL);
+	
+	osync_plugin_set_initialize(plugin, osync_filesync_initialize);
+	osync_plugin_set_finalize(plugin, osync_filesync_finalize);
+	osync_plugin_set_discover(plugin, osync_filesync_discover);
+	
+	osync_plugin_env_register_plugin(env, plugin);
+	osync_plugin_unref(plugin);
+
 	
 	return TRUE;
 	
