@@ -6,316 +6,6 @@
 #include <opensync/opensync-engine.h>
 #include <opensync/opensync-mapping.h>
 
-static void create_random_file(const char *path)
-{
-	char *content = osync_rand_str(g_random_int_range(100, 200));
-	osync_assert(osync_file_write(path, content, strlen(content), 0700, NULL) == TRUE);
-	g_free(content);
-}
-
-int num_client_connected = 0;
-int num_client_main_connected = 0;
-int num_client_disconnected = 0;
-int num_client_main_disconnected = 0;
-int num_client_read = 0;
-int num_client_main_read = 0;
-int num_client_written = 0;
-int num_client_main_written = 0;
-int num_client_errors = 0;
-int num_client_sync_done = 0;
-int num_client_main_sync_done = 0;
-int num_client_discovered = 0;
-
-int num_change_read = 0;
-int num_change_written = 0;
-int num_change_error = 0;
-
-void member_status(OSyncMemberUpdate *status, void *user_data)
-{
-	osync_trace(TRACE_ENTRY, "%s(%p (%i), %p)", __func__, status, status->type, user_data);
-	fail_unless(GINT_TO_POINTER(1) == user_data, NULL);
-	
-	switch (status->type) {
-		case OSYNC_CLIENT_EVENT_CONNECTED:
-			fail_unless(!osync_error_is_set(&(status->error)), NULL);
-			
-			if (status->objtype == NULL) {
-				num_client_main_connected++;
-			} else {
-				fail_unless(!strncmp(status->objtype, "mockobjtype", 11), NULL);
-				num_client_connected++;
-			}
-			
-			break;
-		case OSYNC_CLIENT_EVENT_DISCONNECTED:
-			fail_unless(!osync_error_is_set(&(status->error)), NULL);
-			
-			if (status->objtype == NULL) {
-				num_client_main_disconnected++;
-			} else {
-				fail_unless(!strncmp(status->objtype, "mockobjtype", 11), NULL);
-				num_client_disconnected++;
-			}
-			
-			break;
-		case OSYNC_CLIENT_EVENT_READ:
-			fail_unless(!osync_error_is_set(&(status->error)), NULL);
-			
-			if (status->objtype == NULL) {
-				num_client_main_read++;
-			} else {
-				fail_unless(!strncmp(status->objtype, "mockobjtype", 11), NULL);
-				num_client_read++;
-			}
-			
-			break;
-		case OSYNC_CLIENT_EVENT_WRITTEN:
-			fail_unless(!osync_error_is_set(&(status->error)), NULL);
-			
-			if (status->objtype == NULL) {
-				num_client_main_written++;
-			} else {
-				fail_unless(!strncmp(status->objtype, "mockobjtype", 11), NULL);
-				num_client_written++;
-			}
-			
-			break;
-		case OSYNC_CLIENT_EVENT_ERROR:
-			fail_unless(osync_error_is_set(&(status->error)), NULL);
-			num_client_errors++;
-			break;
-		case OSYNC_CLIENT_EVENT_SYNC_DONE:
-			fail_unless(!osync_error_is_set(&(status->error)), NULL);
-			
-			if (status->objtype == NULL) {
-				num_client_main_sync_done++;
-			} else {
-				fail_unless(!strncmp(status->objtype, "mockobjtype", 11), NULL);
-				num_client_sync_done++;
-			}
-			
-			break;
-		case OSYNC_CLIENT_EVENT_DISCOVERED:
-			fail_unless(!osync_error_is_set(&(status->error)), NULL);
-			num_client_discovered++;
-			break;
-	}
-	
-	osync_trace(TRACE_EXIT, "%s", __func__);
-}
-
-void entry_status(OSyncChangeUpdate *status, void *user_data)
-{
-	osync_trace(TRACE_ENTRY, "%s(%p (%i), %p)", __func__, status, status->type, user_data);
-	
-	switch (status->type) {
-		case OSYNC_CHANGE_EVENT_READ:
-			fail_unless(!osync_error_is_set(&(status->error)), NULL);
-			num_change_read++;
-			break;
-		case OSYNC_CHANGE_EVENT_WRITTEN:
-			fail_unless(!osync_error_is_set(&(status->error)), NULL);
-			num_change_written++;
-			break;
-		case OSYNC_CHANGE_EVENT_ERROR:
-			fail_unless(osync_error_is_set(&(status->error)), NULL);
-			num_change_error++;
-			break;
-	}
-	
-	osync_trace(TRACE_EXIT, "%s", __func__);
-}
-
-int num_engine_connected = 0;
-int num_engine_read = 0;
-int num_engine_written = 0;
-int num_engine_disconnected = 0;
-int num_engine_errors = 0;
-int num_engine_successful = 0;
-int num_engine_end_conflicts = 0;
-int num_engine_prev_unclean = 0;
-int num_engine_sync_done = 0;
-
-void engine_status(OSyncEngineUpdate *status, void *user_data)
-{
-	switch (status->type) {
-		case OSYNC_ENGINE_EVENT_CONNECTED:
-			fail_unless(!osync_error_is_set(&(status->error)), NULL);
-			num_engine_connected++;
-			break;
-		case OSYNC_ENGINE_EVENT_READ:
-			fail_unless(!osync_error_is_set(&(status->error)), NULL);
-			num_engine_read++;
-			break;
-		case OSYNC_ENGINE_EVENT_WRITTEN:
-			fail_unless(!osync_error_is_set(&(status->error)), NULL);
-			num_engine_written++;
-			break;
-		case OSYNC_ENGINE_EVENT_DISCONNECTED:
-			fail_unless(!osync_error_is_set(&(status->error)), NULL);
-			num_engine_disconnected++;
-			break;
-		case OSYNC_ENGINE_EVENT_ERROR:
-			fail_unless(osync_error_is_set(&(status->error)), NULL);
-			num_engine_errors++;
-			break;
-		case OSYNC_ENGINE_EVENT_SUCCESSFUL:
-			fail_unless(!osync_error_is_set(&(status->error)), NULL);
-			num_engine_successful++;
-			break;
-		case OSYNC_ENGINE_EVENT_PREV_UNCLEAN:
-			fail_unless(!osync_error_is_set(&(status->error)), NULL);
-			num_engine_prev_unclean++;
-			break;
-		case OSYNC_ENGINE_EVENT_END_CONFLICTS:
-			fail_unless(!osync_error_is_set(&(status->error)), NULL);
-			num_engine_end_conflicts++;
-			break;
-		case OSYNC_ENGINE_EVENT_SYNC_DONE:
-			fail_unless(!osync_error_is_set(&(status->error)), NULL);
-			num_engine_sync_done++;
-			break;
-	}
-}
-
-int num_mapping_solved = 0;
-int num_mapping_written = 0;
-int num_mapping_errors = 0;
-
-void mapping_status(OSyncMappingUpdate *status, void *user_data)
-{
-	switch (status->type) {
-		case OSYNC_MAPPING_EVENT_SOLVED:
-			fail_unless(!osync_error_is_set(&(status->error)), NULL);
-			num_mapping_solved++;
-			break;
-		/*case OSYNC_MAPPING_EVENT_WRITTEN:
-			fail_unless(!osync_error_is_set(&(status->error)), NULL);
-			num_mapping_written++;
-			break;*/
-		case OSYNC_MAPPING_EVENT_ERROR:
-			fail_unless(osync_error_is_set(&(status->error)), NULL);
-			num_mapping_errors++;
-			break;
-	}
-}
-
-int num_mapping_conflicts = 0;
-
-static void conflict_handler_choose_first(OSyncEngine *engine, OSyncMappingEngine *mapping, void *user_data)
-{
-	osync_trace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, engine, mapping, user_data);
-	
-	num_mapping_conflicts++;
-	fail_unless(osync_mapping_engine_num_changes(mapping) == GPOINTER_TO_INT(user_data), NULL);
-	fail_unless(num_engine_end_conflicts == 0, NULL);
-
-	OSyncChange *change = osync_mapping_engine_member_change(mapping, 1);
-	OSyncError *error = NULL;
-	osync_assert(osync_engine_mapping_solve(engine, mapping, change, &error));
-	osync_assert(error == NULL);
-	
-	osync_trace(TRACE_EXIT, "%s", __func__);
-}
-
-static void conflict_handler_choose_deleted(OSyncEngine *engine, OSyncMappingEngine *mapping, void *user_data)
-{
-	OSyncError *error = NULL;
-	osync_trace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, engine, mapping, user_data);
-	
-	num_mapping_conflicts++;
-	fail_unless(osync_mapping_engine_num_changes(mapping) == GPOINTER_TO_INT(user_data), NULL);
-	fail_unless(num_engine_end_conflicts == 0, NULL);
-
-	int i;
-	for (i = 0; i < osync_mapping_engine_num_changes(mapping); i++) {
-		OSyncChange *change = osync_mapping_engine_nth_change(mapping, i);
-		if (osync_change_get_changetype(change) == OSYNC_CHANGE_TYPE_DELETED) {
-			osync_assert(osync_engine_mapping_solve(engine, mapping, change, &error));
-			osync_assert(error == NULL);
-			
-			osync_trace(TRACE_EXIT, "%s", __func__);
-			return;
-		}
-	}
-	fail(NULL);
-}
-
-static void conflict_handler_duplicate(OSyncEngine *engine, OSyncMappingEngine *mapping, void *user_data)
-{
-	osync_trace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, engine, mapping, user_data);
-	
-	num_mapping_conflicts++;
-	fail_unless(osync_mapping_engine_num_changes(mapping) == GPOINTER_TO_INT(user_data), NULL);
-	fail_unless(num_engine_end_conflicts == 0, NULL);
-	
-	OSyncError *error = NULL;
-	fail_unless(osync_engine_mapping_duplicate(engine, mapping, &error), NULL);
-	
-	osync_trace(TRACE_EXIT, "%s", __func__);
-}
-
-OSyncEngine *gengine = NULL;
-
-static void solve_conflict(OSyncMappingEngine *mapping)
-{
-	sleep(5);
-	
-	OSyncChange *change = osync_mapping_engine_nth_change(mapping, 0);
-	OSyncError *error = NULL;
-	osync_assert(osync_engine_mapping_solve(gengine, mapping, change, &error));
-	osync_assert(error == NULL);
-}
-
-static void conflict_handler_delay(OSyncEngine *engine, OSyncMappingEngine *mapping, void *user_data)
-{
-	osync_trace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, engine, mapping, user_data);
-	
-	num_mapping_conflicts++;
-	fail_unless(osync_mapping_engine_num_changes(mapping) == GPOINTER_TO_INT(user_data), NULL);
-	fail_unless(num_engine_end_conflicts == 0, NULL);
-	gengine = engine;
-	g_thread_create ((GThreadFunc)solve_conflict, mapping, TRUE, NULL);
-	
-	osync_trace(TRACE_EXIT, "%s", __func__);
-}
-
-static void reset_counters()
-{
-	num_client_connected = 0;
-	num_client_main_connected = 0;
-	num_client_disconnected = 0;
-	num_client_main_disconnected = 0;
-	num_client_read = 0;
-	num_client_main_read = 0;
-	num_client_written = 0;
-	num_client_main_written = 0;
-	num_client_errors = 0;
-	num_client_sync_done = 0;
-	num_client_main_sync_done = 0;
-	num_client_discovered = 0;
-	
-	num_change_read = 0;
-	num_change_written = 0;
-	num_change_error = 0;
-	
-	num_engine_connected = 0;
-	num_engine_read = 0;
-	num_engine_written = 0;
-	num_engine_disconnected = 0;
-	num_engine_errors = 0;
-	num_engine_successful = 0;
-	num_engine_end_conflicts = 0;
-	num_engine_prev_unclean = 0;
-	num_engine_sync_done = 0;
-	
-	num_mapping_solved = 0;
-	num_mapping_written = 0;
-	num_mapping_errors = 0;
-	
-	num_mapping_conflicts = 0;
-}
-
 START_TEST (sync_setup_connect)
 {
 	char *testbed = setup_testbed("sync");
@@ -2156,11 +1846,11 @@ START_TEST (sync_large)
 	system("rm -f data1/file7");
 	system("rm -f data2/file7");
 	//delete left, modify right
-	system("rm -f data2/file8");
-	create_random_file("data1/file8");
+	system("rm -f data1/file8");
+	create_random_file("data2/file8");
 	//modify left, delete right
-	create_random_file("data2/file9");
-	system("rm -f data1/file9");
+	create_random_file("data1/file9");
+	system("rm -f data2/file9");
 	
 	
 	fail_unless(osync_engine_synchronize_and_block(engine, &error), NULL);
@@ -2747,6 +2437,205 @@ START_TEST (sync_detect_obj2)
 }
 END_TEST
 
+/* sync_slowsync_connect is intendet to check if a slow-sync got triggered
+   for all members/sinks when the slow-sync got requested during the connect()
+   function of one of the members (e.g. anchor mismatch). Bug Ticket: #538 */
+START_TEST (sync_slowsync_connect)
+{
+	char *testbed = setup_testbed("sync");
+	system("cp testdata data1/testdata");
+	
+	OSyncError *error = NULL;
+	OSyncGroup *group = osync_group_new(&error);
+	fail_unless(group != NULL, NULL);
+	fail_unless(error == NULL, NULL);
+	
+	fail_unless(osync_group_load(group, "configs/group", &error), NULL);
+	fail_unless(error == NULL, NULL);
+	
+	OSyncEngine *engine = osync_engine_new(group, &error);
+	fail_unless(engine != NULL, NULL);
+	fail_unless(error == NULL, NULL);
+	osync_group_unref(group);
+	
+	osync_engine_set_plugindir(engine, testbed);
+	osync_engine_set_formatdir(engine, testbed);
+	
+	osync_engine_set_conflict_callback(engine, conflict_handler_choose_first, GINT_TO_POINTER(1));
+	osync_engine_set_changestatus_callback(engine, entry_status, GINT_TO_POINTER(1));
+	osync_engine_set_mappingstatus_callback(engine, mapping_status, GINT_TO_POINTER(1));
+	osync_engine_set_enginestatus_callback(engine, engine_status, GINT_TO_POINTER(1));
+	osync_engine_set_memberstatus_callback(engine, member_status, GINT_TO_POINTER(1));
+	
+	
+	fail_unless(osync_engine_initialize(engine, &error), NULL);
+	fail_unless(error == NULL, NULL);
+	
+	fail_unless(osync_engine_synchronize_and_block(engine, &error), NULL);
+	fail_unless(error == NULL, NULL);
+	
+	fail_unless(osync_engine_finalize(engine, &error), NULL);
+	fail_unless(error == NULL, NULL);
+	
+	osync_engine_unref(engine);
+	
+	/* Client checks */
+	fail_unless(num_client_connected == 2, NULL);
+	fail_unless(num_client_main_connected == 2, NULL);
+	fail_unless(num_client_read == 2, NULL);
+	fail_unless(num_client_main_read == 2, NULL);
+	fail_unless(num_client_written == 2, NULL);
+	fail_unless(num_client_main_written == 2, NULL);
+	fail_unless(num_client_disconnected == 2, NULL);
+	fail_unless(num_client_main_disconnected == 2, NULL);
+	fail_unless(num_client_errors == 0, NULL);
+	fail_unless(num_client_sync_done == 2, NULL);
+	fail_unless(num_client_main_sync_done == 2, NULL);
+	
+	/* Client checks */
+	fail_unless(num_engine_connected == 1, NULL);
+	fail_unless(num_engine_errors == 0, NULL);
+	fail_unless(num_engine_read == 1, NULL);
+	fail_unless(num_engine_written == 1, NULL);
+	fail_unless(num_engine_sync_done == 1, NULL);
+	fail_unless(num_engine_disconnected == 1, NULL);
+	fail_unless(num_engine_successful == 1, NULL);
+	fail_unless(num_engine_end_conflicts == 1, NULL);
+	fail_unless(num_engine_prev_unclean == 0, NULL);
+
+	/* Change checks */
+	fail_unless(num_change_read == 1, NULL);
+	fail_unless(num_change_written == 1, NULL);
+	fail_unless(num_change_error == 0, NULL);
+
+	/* Mapping checks */
+	fail_unless(num_mapping_solved == 1, NULL);
+	//fail_unless(num_mapping_written == 1, NULL);
+	fail_unless(num_mapping_errors == 0, NULL);
+	fail_unless(num_mapping_conflicts == 0, NULL);
+
+	fail_unless(!system("test \"x$(diff -x \".*\" data1 data2)\" = \"x\""), NULL);
+	
+	char *path = g_strdup_printf("%s/configs/group/archive.db", testbed);
+	OSyncMappingTable *maptable = mappingtable_load(path, "mockobjtype1", 1);
+	g_free(path);
+	check_mapping(maptable, 1, 1, 2, "testdata");
+	check_mapping(maptable, 2, 1, 2, "testdata");
+    osync_mapping_table_close(maptable);
+    osync_mapping_table_unref(maptable);
+    
+	path = g_strdup_printf("%s/configs/group/1/hashtable.db", testbed);
+    OSyncHashTable *table = hashtable_load(path, "mockobjtype1", 1);
+	g_free(path);
+    check_hash(table, "testdata");
+	osync_hashtable_free(table);
+
+	path = g_strdup_printf("%s/configs/group/2/hashtable.db", testbed);
+    table = hashtable_load(path, "mockobjtype1", 1);
+	g_free(path);
+    check_hash(table, "testdata");
+	osync_hashtable_free(table);
+
+	/* 2nd Sync with new path config for #1 member - this is needed
+	 * to trigger a slow-sync within the connect() plugin call. */
+	reset_counters();
+	system("mkdir data3");
+	system("cp configs/new_path.conf configs/group/1/mock-sync.conf");
+	
+	group = osync_group_new(&error);
+	fail_unless(group != NULL, NULL);
+	fail_unless(error == NULL, NULL);
+	
+	fail_unless(osync_group_load(group, "configs/group", &error), NULL);
+	fail_unless(error == NULL, NULL);
+	
+	engine = osync_engine_new(group, &error);
+	fail_unless(engine != NULL, NULL);
+	fail_unless(error == NULL, NULL);
+	osync_group_unref(group);
+	
+	osync_engine_set_plugindir(engine, testbed);
+	osync_engine_set_formatdir(engine, testbed);
+	
+	osync_engine_set_conflict_callback(engine, conflict_handler_choose_first, GINT_TO_POINTER(1));
+	osync_engine_set_changestatus_callback(engine, entry_status, GINT_TO_POINTER(1));
+	osync_engine_set_mappingstatus_callback(engine, mapping_status, GINT_TO_POINTER(1));
+	osync_engine_set_enginestatus_callback(engine, engine_status, GINT_TO_POINTER(1));
+	osync_engine_set_memberstatus_callback(engine, member_status, GINT_TO_POINTER(1));
+	
+	
+	fail_unless(osync_engine_initialize(engine, &error), NULL);
+	fail_unless(error == NULL, NULL);
+	
+	fail_unless(osync_engine_synchronize_and_block(engine, &error), NULL);
+	fail_unless(error == NULL, NULL);
+	
+	fail_unless(osync_engine_finalize(engine, &error), NULL);
+	fail_unless(error == NULL, NULL);
+	
+	osync_engine_unref(engine);
+	
+	/* Client checks */
+	fail_unless(num_client_connected == 2, NULL);
+	fail_unless(num_client_main_connected == 2, NULL);
+	fail_unless(num_client_read == 2, NULL);
+	fail_unless(num_client_main_read == 2, NULL);
+	fail_unless(num_client_written == 2, NULL);
+	fail_unless(num_client_main_written == 2, NULL);
+	fail_unless(num_client_disconnected == 2, NULL);
+	fail_unless(num_client_main_disconnected == 2, NULL);
+	fail_unless(num_client_errors == 0, NULL);
+	fail_unless(num_client_sync_done == 2, NULL);
+	fail_unless(num_client_main_sync_done == 2, NULL);
+	
+	/* Client checks */
+	fail_unless(num_engine_connected == 1, NULL);
+	fail_unless(num_engine_errors == 0, NULL);
+	fail_unless(num_engine_read == 1, NULL);
+	fail_unless(num_engine_written == 1, NULL);
+	fail_unless(num_engine_sync_done == 1, NULL);
+	fail_unless(num_engine_disconnected == 1, NULL);
+	fail_unless(num_engine_successful == 1, NULL);
+	fail_unless(num_engine_end_conflicts == 1, NULL);
+	fail_unless(num_engine_prev_unclean == 0, NULL);
+
+	/* Change checks */
+	fail_unless(num_change_read == 1, NULL);
+	fail_unless(num_change_written == 1, NULL);
+	fail_unless(num_change_error == 0, NULL);
+
+	/* Mapping checks */
+	fail_unless(num_mapping_solved == 1, NULL);
+	//fail_unless(num_mapping_written == 1, NULL);
+	fail_unless(num_mapping_errors == 0, NULL);
+	fail_unless(num_mapping_conflicts == 0, NULL);
+
+	fail_unless(!system("test \"x$(diff -x \".*\" data3 data2)\" = \"x\""), NULL);
+	
+	path = g_strdup_printf("%s/configs/group/archive.db", testbed);
+	maptable = mappingtable_load(path, "mockobjtype1", 1);
+	g_free(path);
+	check_mapping(maptable, 1, 1, 2, "testdata");
+	check_mapping(maptable, 2, 1, 2, "testdata");
+    osync_mapping_table_close(maptable);
+    osync_mapping_table_unref(maptable);
+    
+	path = g_strdup_printf("%s/configs/group/1/hashtable.db", testbed);
+    table = hashtable_load(path, "mockobjtype1", 1);
+	g_free(path);
+    check_hash(table, "testdata");
+	osync_hashtable_free(table);
+
+	path = g_strdup_printf("%s/configs/group/2/hashtable.db", testbed);
+    table = hashtable_load(path, "mockobjtype1", 1);
+	g_free(path);
+    check_hash(table, "testdata");
+	osync_hashtable_free(table);
+
+	destroy_testbed(testbed);
+}
+END_TEST
+
 Suite *env_suite(void)
 {
 	Suite *s = suite_create("Sync");
@@ -2770,6 +2659,8 @@ Suite *env_suite(void)
 	create_case(s, "sync_detect_obj", sync_detect_obj);
 	create_case(s, "sync_detect_obj2", sync_detect_obj2);
 
+	create_case(s, "sync_slowsync_connect", sync_slowsync_connect);
+
 	//stateless sync
 	
 	return s;
@@ -2783,7 +2674,7 @@ int main(void)
 	
 	SRunner *sr;
 	sr = srunner_create(s);
-	srunner_run_all(sr, CK_NORMAL);
+	srunner_run_all(sr, CK_VERBOSE);
 	nf = srunner_ntests_failed(sr);
 	srunner_free(sr);
 	return (nf == 0) ? EXIT_SUCCESS : EXIT_FAILURE;
